#!/usr/bin/env python
#-*- coding: utf-8 -*-

__all__ = [ 'Template' ]

conv = 45.563352491687866

class Template(dict):
    """
This class holds data obtained by the LoProp transformation.
Each template depends on 5 variables.

=========== ============================================ ==========
Variable    Choices                                      Type
=========== ============================================ ==========
Model:      TIP3P/SULACID/PNA                            string
Method:     HF/B3LYP/CAMB3LYP                            string
Basis:      PVDZ/ANOPVDZ/ANOPVTZ                         string
LoProp:     True/False                                   bool
Frequency:  Field :math:`\\omega` with %1.7f format      string
=========== ============================================ ==========

BASIS AVAILABLE


===========              ==============    ===============
Basis                    Element           Basis sets
===========              ==============    ===============
ANOTINY                  H                 2s
                         C/N/O             3s 2p 1d
-----------              --------          -----------
ANOPVDZ                  H                 2s 1s
                         C/N/O             3s 2p 1d
-----------              --------          -----------
ANO631                   H                 2s
                         C/N/O             4s 3p 1d
-----------              --------          -----------
ANO631PGS                H                 3s 1p
                         C/N/O             4s 3p 1d
-----------              --------          -----------
ANOPVTZ                  H                 3s 2p 1d
                         C/N/O             4s 3p 2d 1f
-----------              --------          -----------
ANOTHIRD                 H                 3s 2p 1d
                         C/N/O             5s 4p 3d 2f
-----------              --------          -----------
ANOMAX                   H                 5s 4p 3d
                         C/N/O             5s 4p 3d 2f
===========              ==============    ===============


.. code:: python

    >>> temp = Template().get()
    >>> print temp ("O1", "charge") 
    0.0

    >>> temp = Template().get( model = "TIP3P", method = "HF",
            basis = "ANOPVDZ", dist = True, freq = "0.0"  )
    >>> print temp ("O1", "charge") 
    -0.678

"""


    def get(self, model = "TIP3P_PDB", method = "B3LYP", basis ="ANO631",
            loprop = True, freq = "0.0", nm = False ):
        if freq == "0.0000000":
            freq = "0.0"
        if nm:
            freq = "%.7f" %( conv/ freq )
        if model.lower() == 'tip4p':
            model = 'TIP3P'
        return self[ ( model.upper(), method.upper(), basis.upper(), loprop, freq) ]


    def __init__(self):
# ----------------------------------------------------------------
#
#       All basis for TIP3P model with  HF, B3LYP, CAMB3LYP methods
#
# ----------------------------------------------------------------

#HF anotiny 2s / 3s 2p 1d
        self[("TIP3P", "HF", "ANOTINY", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6470241,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3639924, ],
( 'O1' , 'quadrupole') : [ -3.4930830,  0.0000000,  -0.0000000,  -5.0769431,  -0.0000000,  -4.3212374, ],
( 'O1' , 'alpha' ) : [ 3.7810674,  -0.0000000,  -0.0000000,  3.4476048,  0.0000000,  3.6264429, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -3.6699177,  -0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  -0.4081607,  -0.0000000,  0.7207230, ],
( 'H2' , 'charge') : 0.3235121,
( 'H2' , 'dipole') : [ -0.1616550,  -0.0000000,  -0.1077685, ],
( 'H2' , 'quadrupole') : [ -0.1798713,  0.0000000,  0.2382572,  -0.4749407,  0.0000000,  -0.2907013, ],
( 'H2' , 'alpha' ) : [ 1.4856883,  -0.0000000,  1.1507473,  0.7909575,  -0.0000000,  1.2845498, ],
( 'H2' , 'beta'  ) : [ -5.1163796,  0.0000000,  -5.2884708,  -3.0930268,  -0.0000000,  -5.1136249,  -0.0000000,  -0.9117399,  -0.0000000,  -3.1529317, ],
( 'H3' , 'charge') : 0.3235121,
( 'H3' , 'dipole') : [ 0.1616550,  -0.0000000,  -0.1077685, ],
( 'H3' , 'quadrupole') : [ -0.1798713,  -0.0000000,  -0.2382572,  -0.4749407,  0.0000000,  -0.2907013, ],
( 'H3' , 'alpha' ) : [ 1.4856883,  -0.0000000,  -1.1507473,  0.7909575,  0.0000000,  1.2845498, ],
( 'H3' , 'beta'  ) : [ 5.1163796,  -0.0000000,  -5.2884708,  3.0930268,  0.0000000,  5.1136249,  -0.0000000,  -0.9117399,  -0.0000000,  -3.1529317, ],
}
        self[("TIP3P", "HF", "ANOTINY", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8648128, ],
( 'X', 'quadrupole') : [ -3.8528256,  0.0000000,  0.0000000,  -6.0268245,  0.0000000,  -4.9026400, ],
( 'X', 'alpha' ) : [ 6.7524440,  0.0000000,  0.0000000,  5.0295198,  0.0000000,  6.1955425, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -14.2468593,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -2.2316405,  0.0000000,  -5.5851404, ],
}

#HF ANOPVDZ 2s 1p / 3s 2p 1d
        self[("TIP3P", "HF", "ANOPVDZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6778659,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2980940, ],
( 'O1' , 'quadrupole') : [ -3.5630816,  0.0000000,  0.0000000,  -4.9495746,  -0.0000000,  -4.2491914, ],
( 'O1' , 'alpha' ) : [ 3.8729816,  -0.0000000,  0.0000000,  3.3433254,  0.0000000,  3.7729207, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -4.6978857,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  0.7178860,  -0.0000000,  2.7087304, ],
( 'H2' , 'charge') : 0.3389330,
( 'H2' , 'dipole') : [ -0.1544795,  -0.0000000,  -0.1307689, ],
( 'H2' , 'quadrupole') : [ -0.0953045,  0.0000000,  0.2905635,  -0.4545148,  0.0000000,  -0.2294014, ],
( 'H2' , 'alpha' ) : [ 1.8189638,  -0.0000000,  1.0688151,  1.1983810,  -0.0000000,  1.5440024, ],
( 'H2' , 'beta'  ) : [ -6.9062192,  0.0000000,  -4.8560170,  -4.5256133,  -0.0000000,  -4.8661822,  -0.0000000,  -1.4962497,  -0.0000000,  -4.2866355, ],
( 'H3' , 'charge') : 0.3389330,
( 'H3' , 'dipole') : [ 0.1544795,  -0.0000000,  -0.1307689, ],
( 'H3' , 'quadrupole') : [ -0.0953045,  -0.0000000,  -0.2905635,  -0.4545148,  0.0000000,  -0.2294014, ],
( 'H3' , 'alpha' ) : [ 1.8189638,  -0.0000000,  -1.0688151,  1.1983810,  0.0000000,  1.5440024, ],
( 'H3' , 'beta'  ) : [ 6.9062192,  -0.0000000,  -4.8560170,  4.5256133,  0.0000000,  4.8661822,  -0.0000000,  -1.4962497,  -0.0000000,  -4.2866355, ],

}

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7870603, ],
( 'X', 'quadrupole') : [ -3.7536906,  0.0000000,  0.0000000,  -5.8586042,  0.0000000,  -4.7079942, ],
( 'X', 'alpha' ) : [ 7.5109092,  0.0000000,  0.0000000,  5.7400874,  0.0000000,  6.8609255, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -14.4099197,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -2.2746134,  0.0000000,  -5.8645406, ],
}


#HF ANO631
        self[("TIP3P", "HF", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6638768,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3450971, ],
( 'O1' , 'quadrupole') : [ -3.3441982,  0.0000000,  0.0000000,  -4.8737884,  -0.0000000,  -4.1257814, ],
( 'O1' , 'alpha' ) : [ 3.7832693,  -0.0000000,  0.0000000,  3.9661048,  0.0000000,  3.5266815, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -2.9842960,  0.0000000,  -0.0000000,  0.0000000,  -0.0000000,  1.2674483,  -0.0000000,  2.1673092, ],
( 'H2' , 'charge') : 0.3319384,
( 'H2' , 'dipole') : [ -0.1605791,  -0.0000000,  -0.1129931, ],
( 'H2' , 'quadrupole') : [ -0.1902230,  0.0000000,  0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'H2' , 'alpha' ) : [ 1.5523511,  -0.0000000,  1.1549527,  0.6085970,  -0.0000000,  1.2110419, ],
( 'H2' , 'beta'  ) : [ -4.4682050,  0.0000000,  -4.5590872,  -0.0560177,  0.0000000,  -3.7202959,  -0.0000000,  0.4603997,  -0.0000000,  -2.4041016, ],
( 'H3' , 'charge') : 0.3319384,
( 'H3' , 'dipole') : [ 0.1605791,  -0.0000000,  -0.1129931, ],
( 'H3' , 'quadrupole') : [ -0.1902230,  -0.0000000,  -0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'H3' , 'alpha' ) : [ 1.5523511,  -0.0000000,  -1.1549527,  0.6085970,  0.0000000,  1.2110419, ],
( 'H3' , 'beta'  ) : [ 4.4682050,  -0.0000000,  -4.5590872,  0.0560177,  0.0000000,  3.7202959,  -0.0000000,  0.4603997,  -0.0000000,  -2.4041016, ],

}

        self[("TIP3P", "HF", "ANO631", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8541268, ],
( 'X', 'quadrupole') : [ -3.7246442,  0.0000000,  0.0000000,  -5.7650634,  0.0000000,  -4.6697648, ],
( 'X', 'alpha' ) : [ 6.8879715,  0.0000000,  0.0000000,  5.1832988,  0.0000000,  5.9487653, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.1024704,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  2.1882477,  0.0000000,  -2.6408940, ],
}


#HF maartens loprop 6_31pgp_loprop basis
        self[("TIP3P", "HF", "LOPROP_631PGP", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.7176166,
( 'OW' , 'dipole') : [ 0.0000000,  0.0000000,  0.3137978, ],
( 'OW' , 'quadrupole') : [ -3.4118765,  -0.0000000,  0.0000000,  -4.8259921,  0.0000000,  -4.1830478, ],
( 'OW' , 'alpha' ) : [ 3.6963909,  0.0000000,  -0.0000000,  4.0417104,  -0.0000000,  3.2835090, ],
( 'OW' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -3.1958272,  -0.0000000,  -0.0000000,  -0.0000000,  -0.0000000,  -0.1723118,  -0.0000000,  2.0471428, ],
( 'HW1', 'charge') : 0.3588083,
( 'HW1', 'dipole') : [ -0.1420943,  -0.0000000,  -0.0959243, ],
( 'HW1', 'quadrupole') : [ -0.1855110,  0.0000000,  0.1973563,  -0.4443793,  0.0000000,  -0.2672644, ],
( 'HW1', 'alpha' ) : [ 1.7153492,  0.0000000,  1.1839364,  0.5171026,  0.0000000,  1.2112600, ],
( 'HW1', 'beta'  ) : [ -5.9541712,  -0.0000000,  -4.8405727,  0.4204104,  -0.0000000,  -4.0791013,  -0.0000000,  0.2536120,  -0.0000000,  -3.0359090, ],
( 'HW2', 'charge') : 0.3588083,
( 'HW2', 'dipole') : [ 0.1420943,  0.0000000,  -0.0959243, ],
( 'HW2', 'quadrupole') : [ -0.1855110,  0.0000000,  -0.1973563,  -0.4443793,  -0.0000000,  -0.2672644, ],
( 'HW2', 'alpha' ) : [ 1.7153492,  0.0000000,  -1.1839364,  0.5171026,  -0.0000000,  1.2112600, ],
( 'HW2', 'beta'  ) : [ 5.9541712,  0.0000000,  -4.8405727,  -0.4204104,  -0.0000000,  4.0791013,  0.0000000,  0.2536120,  0.0000000,  -3.0359090, ],

}
        self[("TIP3P", "HF", "LOPROP_631PGP", False, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.9164635, ],
( 'X', 'quadrupole') : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'X', 'alpha' ) : [ 7.1270893,  0.0000000,  0.0000000,  5.0759156,  0.0000000,  5.7060290, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.8769726,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.3349122,  0.0000000,  -4.0246752, ],
}


#HF ANO631PGS 3s 1p H 4s 3p 1d O 
        self[("TIP3P", "HF", "ANO631PGS", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6895738,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2987928, ],
( 'O1' , 'quadrupole') : [ -3.4957777,  0.0000000,  0.0000000,  -4.8306229,  -0.0000000,  -4.1670735, ],
( 'O1' , 'alpha' ) : [ 4.0783710,  -0.0000000,  -0.0000000,  4.1511835,  0.0000000,  3.7659318, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -3.4886293,  -0.0000000,  0.0000000,  0.0000000,  -0.0000000,  2.0647400,  -0.0000000,  4.0469147, ],
( 'H2' , 'charge') : 0.3447869,
( 'H2' , 'dipole') : [ -0.1464682,  -0.0000000,  -0.1291114, ],
( 'H2' , 'quadrupole') : [ -0.1069528,  0.0000000,  0.2624023,  -0.4242104,  0.0000000,  -0.2041291, ],
( 'H2' , 'alpha' ) : [ 1.9342310,  -0.0000000,  1.1331330,  1.0378576,  -0.0000000,  1.5389391, ],
( 'H2' , 'beta'  ) : [ -7.1813225,  0.0000000,  -4.2470261,  -1.8147601,  -0.0000000,  -4.0745831,  -0.0000000,  -1.0452487,  -0.0000000,  -4.3407218, ],
( 'H3' , 'charge') : 0.3447869,
( 'H3' , 'dipole') : [ 0.1464682,  -0.0000000,  -0.1291114, ],
( 'H3' , 'quadrupole') : [ -0.1069528,  -0.0000000,  -0.2624023,  -0.4242104,  0.0000000,  -0.2041291, ],
( 'H3' , 'alpha' ) : [ 1.9342310,  -0.0000000,  -1.1331330,  1.0378576,  0.0000000,  1.5389391, ],
( 'H3' , 'beta'  ) : [ 7.1813225,  -0.0000000,  -4.2470261,  1.8147601,  0.0000000,  4.0745831,  -0.0000000,  -1.0452487,  -0.0000000,  -4.3407218, ],

}

        self[("TIP3P", "HF", "ANO631PGS", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8040365, ],
( 'X', 'quadrupole') : [ -3.7096833,  0.0000000,  0.0000000,  -5.6790437,  0.0000000,  -4.5753317, ],
( 'X', 'alpha' ) : [ 7.9468330,  0.0000000,  0.0000000,  6.2268987,  0.0000000,  6.8438100, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -11.9826815,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -0.0257574,  0.0000000,  -4.6345289, ],
}

#ANOPVTZ 3s 2p 1d  // 4s 3p 2d 1f
        self[("TIP3P", "HF", "ANOPVTZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6892422,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2685363, ],
( 'O1' , 'quadrupole') : [ -3.5262885,  0.0000000,  -0.0000000,  -4.8331961,  0.0000000,  -4.1403834, ],
( 'O1' , 'alpha' ) : [ 4.9997510,  -0.0000000,  0.0000000,  5.4246659,  -0.0000000,  5.2138787, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -2.7146636,  -0.0000000,  0.0000000,  0.0000000,  -0.0000000,  2.2095480,  -0.0000000,  6.4295950, ],
( 'H2' , 'charge') : 0.3446211,
( 'H2' , 'dipole') : [ -0.1490874,  -0.0000000,  -0.1284798, ],
( 'H2' , 'quadrupole') : [ -0.1052364,  0.0000000,  0.2482146,  -0.4141248,  -0.0000000,  -0.2329461, ],
( 'H2' , 'alpha' ) : [ 2.0666708,  -0.0000000,  1.1100585,  1.1880849,  -0.0000000,  1.6143906, ],
( 'H2' , 'beta'  ) : [ -10.2518876,  0.0000000,  -4.6812099,  -3.0678449,  -0.0000000,  -4.7118335,  0.0000000,  -1.2056966,  0.0000000,  -5.8895650, ],
( 'H3' , 'charge') : 0.3446211,
( 'H3' , 'dipole') : [ 0.1490874,  -0.0000000,  -0.1284798, ],
( 'H3' , 'quadrupole') : [ -0.1052364,  -0.0000000,  -0.2482146,  -0.4141248,  -0.0000000,  -0.2329461, ],
( 'H3' , 'alpha' ) : [ 2.0666708,  -0.0000000,  -1.1100585,  1.1880849,  0.0000000,  1.6143906, ],
( 'H3' , 'beta'  ) : [ 10.2518876,  -0.0000000,  -4.6812099,  3.0678449,  0.0000000,  4.7118335,  0.0000000,  -1.2056966,  -0.0000000,  -5.8895650, ],

}

        self[("TIP3P", "HF", "ANOPVTZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7746761, ],
( 'X', 'quadrupole') : [ -3.7367613,  0.0000000,  0.0000000,  -5.6614457,  0.0000000,  -4.6062756, ],
( 'X', 'alpha' ) : [ 9.1330926,  0.0000000,  0.0000000,  7.8008357,  0.0000000,  8.4426599, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.0770834,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -0.2018452,  0.0000000,  -5.3495350, ],
}

#ANOTHIRD 3s 2p 1d  // 5s 4p 3d 2f
        self[("TIP3P", "HF", "ANOTHIRD", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6913350,
( 'O1' , 'dipole') : [ -0.0000000,  -0.0000000,  0.2704873, ],
( 'O1' , 'quadrupole') : [ -3.5266711,  0.0000000,  0.0000000,  -4.7495966,  -0.0000000,  -4.1235067, ],
( 'O1' , 'alpha' ) : [ 5.0697756,  -0.0000000,  -0.0000000,  5.4835892,  0.0000000,  5.2984561, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -1.6383332,  -0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  1.7748893,  0.0000000,  5.7259407, ],
( 'H2' , 'charge') : 0.3456675,
( 'H2' , 'dipole') : [ -0.1500331,  -0.0000000,  -0.1295782, ],
( 'H2' , 'quadrupole') : [ -0.0963066,  -0.0000000,  0.2632036,  -0.4339479,  0.0000000,  -0.2259291, ],
( 'H2' , 'alpha' ) : [ 2.0354892,  -0.0000000,  1.0687339,  1.1591779,  0.0000000,  1.5591595, ],
( 'H2' , 'beta'  ) : [ -9.9209332,  0.0000000,  -4.5082306,  -2.4147116,  -0.0000000,  -4.5013744,  0.0000000,  -0.9367662,  0.0000000,  -5.6554629, ],
( 'H3' , 'charge') : 0.3456675,
( 'H3' , 'dipole') : [ 0.1500331,  -0.0000000,  -0.1295782, ],
( 'H3' , 'quadrupole') : [ -0.0963066,  0.0000000,  -0.2632036,  -0.4339479,  0.0000000,  -0.2259291, ],
( 'H3' , 'alpha' ) : [ 2.0354892,  -0.0000000,  -1.0687339,  1.1591779,  0.0000000,  1.5591595, ],
( 'H3' , 'beta'  ) : [ 9.9209332,  -0.0000000,  -4.5082306,  2.4147116,  0.0000000,  4.5013744,  -0.0000000,  -0.9367662,  -0.0000000,  -5.6554629, ],

}
        self[("TIP3P", "HF", "ANOTHIRD", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7767473, ],
( 'X', 'quadrupole') : [ -3.7192843,  0.0000000,  0.0000000,  -5.6174924,  0.0000000,  -4.5753649, ],
( 'X', 'alpha' ) : [ 9.1407540,  0.0000000,  0.0000000,  7.8019450,  0.0000000,  8.4167751, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -10.6547944,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -0.0986431,  0.0000000,  -5.5849851, ],
}

#ANOMAX 5s 4p 3d  // 5s 4p 3d 2f
        self[("TIP3P", "HF", "ANOMAX", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6897309,
( 'O1' , 'dipole') : [ -0.0000000,  -0.0000000,  0.2880543, ],
( 'O1' , 'quadrupole') : [ -3.5331728,  -0.0000000,  0.0000000,  -4.7737869,  -0.0000000,  -4.1128362, ],
( 'O1' , 'alpha' ) : [ 5.0029238,  0.0000000,  -0.0000000,  5.2577066,  -0.0000000,  5.0449037, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  0.0000000,  -2.8880370,  -0.0000000,  -0.0000000,  -0.0000000,  -0.0000000,  1.1359698,  -0.0000000,  4.9235339, ],
( 'H2' , 'charge') : 0.3448654,
( 'H2' , 'dipole') : [ -0.1467189,  0.0000000,  -0.1368937, ],
( 'H2' , 'quadrupole') : [ -0.0979701,  0.0000000,  0.2701728,  -0.4179040,  -0.0000000,  -0.2127527, ],
( 'H2' , 'alpha' ) : [ 2.0931681,  -0.0000000,  1.0715637,  1.2785932,  0.0000000,  1.7060547, ],
( 'H2' , 'beta'  ) : [ -9.8172948,  -0.0000000,  -3.7567891,  -2.6742121,  -0.0000000,  -3.9719527,  0.0000000,  -0.9639646,  0.0000000,  -5.2702330, ],
( 'H3' , 'charge') : 0.3448654,
( 'H3' , 'dipole') : [ 0.1467189,  -0.0000000,  -0.1368937, ],
( 'H3' , 'quadrupole') : [ -0.0979701,  0.0000000,  -0.2701728,  -0.4179040,  0.0000000,  -0.2127527, ],
( 'H3' , 'alpha' ) : [ 2.0931681,  -0.0000000,  -1.0715637,  1.2785932,  0.0000000,  1.7060547, ],
( 'H3' , 'beta'  ) : [ 9.8172948,  -0.0000000,  -3.7567891,  2.6742121,  0.0000000,  3.9719527,  -0.0000000,  -0.9639646,  -0.0000000,  -5.2702330, ],

}
        self[("TIP3P", "HF", "ANOMAX", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : -0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7779072, ],
( 'X', 'quadrupole') : [ -3.7291130,  0.0000000,  0.0000000,  -5.6095949,  0.0000000,  -4.5383416, ],
( 'X', 'alpha' ) : [ 9.1892600,  0.0000000,  0.0000000,  7.8148930,  0.0000000,  8.4570131, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -10.4016152,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -0.7919594,  0.0000000,  -5.6169321, ],
}

#############################
#    B3LYP START 
#############################

        self[("TIP3P", "B3LYP", "ANOTINY", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6522154,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3432497, ],
( 'O1' , 'quadrupole') : [ -3.4835132,  0.0000000,  -0.0000000,  -5.0686935,  -0.0000000,  -4.3139823, ],
( 'O1' , 'alpha' ) : [ 3.8684242,  -0.0000000,  -0.0000000,  3.7703460,  0.0000000,  3.8122982, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -4.4758583,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -2.8314952,  -0.0000000,  -1.6305528, ],
( 'H2' , 'charge') : 0.3261077,
( 'H2' , 'dipole') : [ -0.1663015,  -0.0000000,  -0.1138122, ],
( 'H2' , 'quadrupole') : [ -0.1737843,  0.0000000,  0.2373289,  -0.4674651,  0.0000000,  -0.2826679, ],
( 'H2' , 'alpha' ) : [ 1.4828143,  -0.0000000,  1.1788253,  0.9929530,  -0.0000000,  1.3768020, ],
( 'H2' , 'beta'  ) : [ -5.8027975,  0.0000000,  -6.1776205,  -5.7164792,  -0.0000000,  -6.5944833,  -0.0000000,  -2.4081383,  -0.0000000,  -4.4208011, ],
( 'H3' , 'charge') : 0.3261077,
( 'H3' , 'dipole') : [ 0.1663015,  -0.0000000,  -0.1138122, ],
( 'H3' , 'quadrupole') : [ -0.1737843,  -0.0000000,  -0.2373289,  -0.4674651,  0.0000000,  -0.2826679, ],
( 'H3' , 'alpha' ) : [ 1.4828143,  -0.0000000,  -1.1788253,  0.9929530,  0.0000000,  1.3768020, ],
( 'H3' , 'beta'  ) : [ 5.8027975,  -0.0000000,  -6.1776205,  5.7164792,  0.0000000,  6.5944833,  -0.0000000,  -2.4081383,  -0.0000000,  -4.4208011, ],
}
        self[("TIP3P", "B3LYP", "ANOTINY", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8377302, ],
( 'X', 'quadrupole') : [ -3.8310818,  0.0000000,  0.0000000,  -6.0036237,  0.0000000,  -4.8793181, ],
( 'X', 'alpha' ) : [ 6.8340528,  0.0000000,  0.0000000,  5.7562520,  0.0000000,  6.5659022, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -16.8310993,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -7.6477718,  0.0000000,  -10.4721550, ],
}

#B3LYP ANOPVDZ 2s 1p / 3s 2p 1d
        self[("TIP3P", "B3LYP", "ANOPVDZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6762193,
( 'O1' , 'dipole') : [ -0.0000000,  -0.0000000,  0.2816494, ],
( 'O1' , 'quadrupole') : [ -3.5497905,  0.0000000,  0.0000000,  -4.9233261,  -0.0000000,  -4.2373182, ],
( 'O1' , 'alpha' ) : [ 4.0263544,  -0.0000000,  0.0000000,  3.6164177,  0.0000000,  4.0083339, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -5.5799503,  0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  -0.7812068,  -0.0000000,  1.2432572, ],
( 'H2' , 'charge') : 0.3381097,
( 'H2' , 'dipole') : [ -0.1661926,  -0.0000000,  -0.1421301, ],
( 'H2' , 'quadrupole') : [ -0.1043903,  0.0000000,  0.2838626,  -0.4538244,  0.0000000,  -0.2337682, ],
( 'H2' , 'alpha' ) : [ 1.8822333,  -0.0000000,  1.1274826,  1.3773563,  -0.0000000,  1.6783550, ],
( 'H2' , 'beta'  ) : [ -7.4631853,  0.0000000,  -5.5560062,  -6.7839598,  -0.0000000,  -6.1113523,  0.0000000,  -2.5366508,  0.0000000,  -5.2932350, ],
( 'H3' , 'charge') : 0.3381097,
( 'H3' , 'dipole') : [ 0.1661926,  -0.0000000,  -0.1421301, ],
( 'H3' , 'quadrupole') : [ -0.1043903,  -0.0000000,  -0.2838626,  -0.4538244,  -0.0000000,  -0.2337682, ],
( 'H3' , 'alpha' ) : [ 1.8822333,  -0.0000000,  -1.1274826,  1.3773563,  0.0000000,  1.6783550, ],
( 'H3' , 'beta'  ) : [ 7.4631853,  -0.0000000,  -5.5560062,  6.7839598,  0.0000000,  6.1113523,  0.0000000,  -2.5366508,  -0.0000000,  -5.2932350, ],

}

        self[("TIP3P", "B3LYP", "ANOPVDZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7460702, ],
( 'X', 'quadrupole') : [ -3.7585711,  0.0000000,  0.0000000,  -5.8309749,  0.0000000,  -4.7048546, ],
( 'X', 'alpha' ) : [ 7.7908210,  0.0000000,  0.0000000,  6.3711303,  0.0000000,  7.3650439, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -16.6919627,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -5.8545084,  0.0000000,  -9.3432128, ],
}


#B3LYP ANO631
        self[("TIP3P", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6622867,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3427575, ],
( 'O1' , 'quadrupole') : [ -3.4641851,  0.0000000,  -0.0000000,  -5.1250616,  -0.0000000,  -4.3163864, ],
( 'O1' , 'alpha' ) : [ 4.1057421,  -0.0000000,  0.0000000,  4.7922852,  0.0000000,  4.0191241, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -3.3316177,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -0.3221599,  -0.0000000,  0.7913706, ],
( 'H2' , 'charge') : 0.3311434,
( 'H2' , 'dipole') : [ -0.1661726,  -0.0000000,  -0.1162879, ],
( 'H2' , 'quadrupole') : [ -0.1963880,  0.0000000,  0.2158936,  -0.4475531,  -0.0000000,  -0.2780105, ],
( 'H2' , 'alpha' ) : [ 1.5380242,  -0.0000000,  1.1976512,  0.9066088,  -0.0000000,  1.3713771, ],
( 'H2' , 'beta'  ) : [ -4.5213728,  0.0000000,  -5.0806123,  -1.3549385,  0.0000000,  -4.8336536,  0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],
( 'H3' , 'charge') : 0.3311434,
( 'H3' , 'dipole') : [ 0.1661726,  -0.0000000,  -0.1162879, ],
( 'H3' , 'quadrupole') : [ -0.1963880,  -0.0000000,  -0.2158936,  -0.4475531,  0.0000000,  -0.2780105, ],
( 'H3' , 'alpha' ) : [ 1.5380242,  -0.0000000,  -1.1976512,  0.9066088,  0.0000000,  1.3713771, ],
( 'H3' , 'beta'  ) : [ 4.5213728,  -0.0000000,  -5.0806123,  1.3549385,  0.0000000,  4.8336536,  -0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],
}

        self[("TIP3P", "B3LYP", "ANO631", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434372, ],
( 'X', 'quadrupole') : [ -3.8569611,  0.0000000,  0.0000000,  -6.0201678,  0.0000000,  -4.8724074, ],
( 'X', 'alpha' ) : [ 7.1817905,  0.0000000,  0.0000000,  6.6055028,  0.0000000,  6.7618783, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.4928423,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -1.2485065,  0.0000000,  -6.1670414, ],
}





#B3LYP ANO631PGS 3s 1p H 4s 3p 1d O 
        self[("TIP3P", "B3LYP", "ANO631PGS", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6832933,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2947422, ],
( 'O1' , 'quadrupole') : [ -3.6074375,  0.0000000,  0.0000000,  -5.0486697,  -0.0000000,  -4.3392984, ],
( 'O1' , 'alpha' ) : [ 4.4590259,  -0.0000000,  -0.0000000,  5.1537101,  0.0000000,  4.3083768, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -3.5402276,  -0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  0.2289039,  -0.0000000,  3.1687540, ],
( 'H2' , 'charge') : 0.3416467,
( 'H2' , 'dipole') : [ -0.1596457,  -0.0000000,  -0.1402509, ],
( 'H2' , 'quadrupole') : [ -0.1086712,  0.0000000,  0.2674438,  -0.4268586,  0.0000000,  -0.2079616, ],
( 'H2' , 'alpha' ) : [ 2.0129388,  -0.0000000,  1.2121085,  1.3456393,  0.0000000,  1.7539835, ],
( 'H2' , 'beta'  ) : [ -7.6426337,  0.0000000,  -4.9118411,  -3.7355877,  -0.0000000,  -5.4559910,  0.0000000,  -2.2067809,  -0.0000000,  -5.5182232, ],
( 'H3' , 'charge') : 0.3416467,
( 'H3' , 'dipole') : [ 0.1596457,  -0.0000000,  -0.1402509, ],
( 'H3' , 'quadrupole') : [ -0.1086712,  -0.0000000,  -0.2674438,  -0.4268586,  0.0000000,  -0.2079616, ],
( 'H3' , 'alpha' ) : [ 2.0129388,  -0.0000000,  -1.2121085,  1.3456393,  0.0000000,  1.7539835, ],
( 'H3' , 'beta'  ) : [ 7.6426337,  -0.0000000,  -4.9118411,  3.7355877,  0.0000000,  5.4559910,  0.0000000,  -2.2067809,  -0.0000000,  -5.5182232, ],

}
        self[("TIP3P", "B3LYP", "ANO631PGS", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7707535, ],
( 'X', 'quadrupole') : [ -3.8247799,  0.0000000,  0.0000000,  -5.9023869,  0.0000000,  -4.7552216, ],
( 'X', 'alpha' ) : [ 8.4849035,  0.0000000,  0.0000000,  7.8449887,  0.0000000,  7.8163438, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.3639098,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -4.1846579,  0.0000000,  -7.8676924, ],
}

#ANOPVTZ 3s 2p 1d  // 4s 3p 2d 1f
        self[("TIP3P", "B3LYP", "ANOPVTZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6829741,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2543387, ],
( 'O1' , 'quadrupole') : [ -3.6214727,  0.0000000,  0.0000000,  -5.0388992,  -0.0000000,  -4.2804535, ],
( 'O1' , 'alpha' ) : [ 5.5872069,  -0.0000000,  -0.0000000,  6.5486692,  0.0000000,  6.1107626, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -3.6771088,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  1.2004626,  -0.0000000,  7.1205884, ],
( 'H2' , 'charge') : 0.3414870,
( 'H2' , 'dipole') : [ -0.1650532,  -0.0000000,  -0.1411296, ],
( 'H2' , 'quadrupole') : [ -0.1152047,  0.0000000,  0.2426231,  -0.4251253,  0.0000000,  -0.2552393, ],
( 'H2' , 'alpha' ) : [ 2.2449679,  -0.0000000,  1.2263998,  1.5358326,  -0.0000000,  1.8874680, ],
( 'H2' , 'beta'  ) : [ -12.0364919,  0.0000000,  -5.7321131,  -5.7458600,  0.0000000,  -6.1990071,  -0.0000000,  -2.3342786,  0.0000000,  -7.5321492, ],
( 'H3' , 'charge') : 0.3414870,
( 'H3' , 'dipole') : [ 0.1650532,  -0.0000000,  -0.1411296, ],
( 'H3' , 'quadrupole') : [ -0.1152047,  -0.0000000,  -0.2426231,  -0.4251253,  0.0000000,  -0.2552393, ],
( 'H3' , 'alpha' ) : [ 2.2449679,  -0.0000000,  -1.2263998,  1.5358326,  0.0000000,  1.8874680, ],
( 'H3' , 'beta'  ) : [ 12.0364919,  -0.0000000,  -5.7321131,  5.7458600,  0.0000000,  6.1990071,  -0.0000000,  -2.3342786,  -0.0000000,  -7.5321492, ],

}

        self[("TIP3P", "B3LYP", "ANOPVTZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : -0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7282390, ],
( 'X', 'quadrupole') : [ -3.8518821,  0.0000000,  0.0000000,  -5.8891498,  0.0000000,  -4.7909321, ],
( 'X', 'alpha' ) : [ 10.0771427,  0.0000000,  0.0000000,  9.6203344,  0.0000000,  9.8856986, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -15.1413350,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -3.4680946,  0.0000000,  -7.9437100, ],
}

#ANOTHIRD 3s 2p 1d  // 5s 4p 3d 2f
        self[("TIP3P", "B3LYP", "ANOTHIRD", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6848896,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2518205, ],
( 'O1' , 'quadrupole') : [ -3.5987886,  0.0000000,  -0.0000000,  -4.9028657,  -0.0000000,  -4.2235558, ],
( 'O1' , 'alpha' ) : [ 5.6469368,  -0.0000000,  0.0000000,  6.5395679,  0.0000000,  6.1128550, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -2.2250489,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.8924470,  0.0000000,  5.8346238, ],
( 'H2' , 'charge') : 0.3424448,
( 'H2' , 'dipole') : [ -0.1651949,  -0.0000000,  -0.1416440, ],
( 'H2' , 'quadrupole') : [ -0.1080099,  0.0000000,  0.2566313,  -0.4467974,  -0.0000000,  -0.2482493, ],
( 'H2' , 'alpha' ) : [ 2.1947925,  -0.0000000,  1.1622453,  1.4530967,  -0.0000000,  1.7821075, ],
( 'H2' , 'beta'  ) : [ -11.6752911,  0.0000000,  -5.4489851,  -4.0300024,  -0.0000000,  -5.6238216,  0.0000000,  -1.6912899,  0.0000000,  -7.1764461, ],
( 'H3' , 'charge') : 0.3424448,
( 'H3' , 'dipole') : [ 0.1651949,  -0.0000000,  -0.1416440, ],
( 'H3' , 'quadrupole') : [ -0.1080099,  -0.0000000,  -0.2566313,  -0.4467974,  0.0000000,  -0.2482493, ],
( 'H3' , 'alpha' ) : [ 2.1947925,  -0.0000000,  -1.1622453,  1.4530967,  -0.0000000,  1.7821075, ],
( 'H3' , 'beta'  ) : [ 11.6752911,  -0.0000000,  -5.4489851,  4.0300024,  0.0000000,  5.6238216,  -0.0000000,  -1.6912899,  -0.0000000,  -7.1764461, ],
}
        self[("TIP3P", "B3LYP", "ANOTHIRD", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7268128, ],
( 'X', 'quadrupole') : [ -3.8148084,  0.0000000,  0.0000000,  -5.7964605,  0.0000000,  -4.7200544, ],
( 'X', 'alpha' ) : [ 10.0365218,  0.0000000,  0.0000000,  9.4457613,  0.0000000,  9.6770700, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.1230191,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -2.4901328,  0.0000000,  -8.5182684, ],
}

#ANOMAX 5s 4p 3d  // 5s 4p 3d 2f
        self[("TIP3P", "B3LYP", "ANOMAX", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6841714,
( 'O1' , 'dipole') : [ -0.0000000,  0.0000000,  0.2657451, ],
( 'O1' , 'quadrupole') : [ -3.6019369,  -0.0000000,  0.0000000,  -4.9156641,  0.0000000,  -4.1979043, ],
( 'O1' , 'alpha' ) : [ 5.6510139,  0.0000000,  -0.0000000,  6.3261916,  0.0000000,  5.8501536, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  0.0000000,  -3.9372857,  -0.0000000,  0.0000000,  -0.0000000,  0.0000000,  -0.1416461,  -0.0000000,  5.2589806, ],
( 'H2' , 'charge') : 0.3420857,
( 'H2' , 'dipole') : [ -0.1583993,  -0.0000000,  -0.1478315, ],
( 'H2' , 'quadrupole') : [ -0.1211319,  0.0000000,  0.2591625,  -0.4386486,  -0.0000000,  -0.2452033, ],
( 'H2' , 'alpha' ) : [ 2.2762024,  -0.0000000,  1.1670150,  1.6150062,  -0.0000000,  1.9774682, ],
( 'H2' , 'beta'  ) : [ -12.0934773,  -0.0000000,  -4.7430352,  -4.5448054,  -0.0000000,  -4.9917258,  -0.0000000,  -1.9957538,  0.0000000,  -7.0290628, ],
( 'H3' , 'charge') : 0.3420857,
( 'H3' , 'dipole') : [ 0.1583993,  -0.0000000,  -0.1478315, ],
( 'H3' , 'quadrupole') : [ -0.1211319,  0.0000000,  -0.2591625,  -0.4386486,  0.0000000,  -0.2452033, ],
( 'H3' , 'alpha' ) : [ 2.2762024,  -0.0000000,  -1.1670150,  1.6150062,  -0.0000000,  1.9774682, ],
( 'H3' , 'beta'  ) : [ 12.0934773,  -0.0000000,  -4.7430352,  4.5448054,  0.0000000,  4.9917258,  -0.0000000,  -1.9957538,  -0.0000000,  -7.0290628, ],

}
        self[("TIP3P", "B3LYP", "ANOMAX", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7275673, ],
( 'X', 'quadrupole') : [ -3.8442007,  0.0000000,  0.0000000,  -5.7929613,  0.0000000,  -4.6883109, ],
( 'X', 'alpha' ) : [ 10.2034187,  0.0000000,  0.0000000,  9.5562040,  0.0000000,  9.8050900, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.4233561,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -4.1331537,  0.0000000,  -8.7991450, ],
}

##########
#       ANOTINY -> ANOMAX for CAMB3lYP static TIP3P water model
#        ----  CAMB3LYP  ----
##########
#ANOTINY 2s / 3s 2p 1d
        self[("TIP3P", "CAMB3LYP", "ANOTINY", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6535307,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3497784, ],
( 'O1' , 'quadrupole') : [ -3.4832507,  0.0000000,  -0.0000000,  -5.0722984,  -0.0000000,  -4.3166089, ],
( 'O1' , 'alpha' ) : [ 3.8445784,  -0.0000000,  -0.0000000,  3.7257924,  0.0000000,  3.7741939, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -4.0483347,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -2.5395898,  -0.0000000,  -1.2817062, ],
( 'H2' , 'charge') : 0.3267654,
( 'H2' , 'dipole') : [ -0.1644427,  -0.0000000,  -0.1109200, ],
( 'H2' , 'quadrupole') : [ -0.1776832,  0.0000000,  0.2344402,  -0.4695500,  0.0000000,  -0.2878080, ],
( 'H2' , 'alpha' ) : [ 1.4700908,  -0.0000000,  1.1591789,  0.9629183,  0.0000000,  1.3565711, ],
( 'H2' , 'beta'  ) : [ -5.5439345,  0.0000000,  -5.8265905,  -5.2660344,  -0.0000000,  -6.2415449,  -0.0000000,  -2.2062353,  -0.0000000,  -4.1967640, ],
( 'H3' , 'charge') : 0.3267654,
( 'H3' , 'dipole') : [ 0.1644427,  -0.0000000,  -0.1109200, ],
( 'H3' , 'quadrupole') : [ -0.1776832,  -0.0000000,  -0.2344402,  -0.4695500,  0.0000000,  -0.2878080, ],
( 'H3' , 'alpha' ) : [ 1.4700908,  -0.0000000,  -1.1591789,  0.9629183,  0.0000000,  1.3565711, ],
( 'H3' , 'beta'  ) : [ 5.5439345,  -0.0000000,  -5.8265905,  5.2660344,  0.0000000,  6.2415449,  0.0000000,  -2.2062353,  -0.0000000,  -4.1967640, ],
}
        self[("TIP3P", "CAMB3LYP", "ANOTINY", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8514996, ],
( 'X', 'quadrupole') : [ -3.8386171,  0.0000000,  0.0000000,  -6.0113984,  0.0000000,  -4.8922249, ],
( 'X', 'alpha' ) : [ 6.7847600,  0.0000000,  0.0000000,  5.6516290,  0.0000000,  6.4873361, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -15.7015157,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -6.9520604,  0.0000000,  -9.6752342, ],
}

#CAMB3LYP ANOPVDZ 2s 1p / 3s 2p 1d
        self[("TIP3P", "CAMB3LYP", "ANOPVDZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6777224,
( 'O1' , 'dipole') : [ -0.0000000,  -0.0000000,  0.2886013, ],
( 'O1' , 'quadrupole') : [ -3.5496408,  0.0000000,  0.0000000,  -4.9288796,  -0.0000000,  -4.2408401, ],
( 'O1' , 'alpha' ) : [ 3.9966157,  -0.0000000,  0.0000000,  3.5788289,  0.0000000,  3.9707146, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -5.1779325,  -0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  -0.6120297,  -0.0000000,  1.4431810, ],
( 'H2' , 'charge') : 0.3388612,
( 'H2' , 'dipole') : [ -0.1642259,  -0.0000000,  -0.1388065, ],
( 'H2' , 'quadrupole') : [ -0.1084691,  0.0000000,  0.2812564,  -0.4567076,  0.0000000,  -0.2389824, ],
( 'H2' , 'alpha' ) : [ 1.8642911,  0.0000000,  1.1072580,  1.3451660,  0.0000000,  1.6541073, ],
( 'H2' , 'beta'  ) : [ -7.2346721,  0.0000000,  -5.2730663,  -6.3359031,  -0.0000000,  -5.8093727,  0.0000000,  -2.3811071,  -0.0000000,  -5.1075037, ],
( 'H3' , 'charge') : 0.3388612,
( 'H3' , 'dipole') : [ 0.1642259,  -0.0000000,  -0.1388065, ],
( 'H3' , 'quadrupole') : [ -0.1084691,  -0.0000000,  -0.2812564,  -0.4567076,  -0.0000000,  -0.2389824, ],
( 'H3' , 'alpha' ) : [ 1.8642911,  -0.0000000,  -1.1072580,  1.3451660,  0.0000000,  1.6541073, ],
( 'H3' , 'beta'  ) : [ 7.2346721,  -0.0000000,  -5.2730663,  6.3359031,  0.0000000,  5.8093727,  -0.0000000,  -2.3811071,  -0.0000000,  -5.1075037, ],
}

        self[("TIP3P", "CAMB3LYP", "ANOPVDZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7613334, ],
( 'X', 'quadrupole') : [ -3.7665790,  0.0000000,  0.0000000,  -5.8422948,  0.0000000,  -4.7188049, ],
( 'X', 'alpha' ) : [ 7.7251979,  0.0000000,  0.0000000,  6.2691609,  0.0000000,  7.2789292, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -15.7240651,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -5.3742439,  0.0000000,  -8.7718264, ],
}

#CAMB3LYP ANO631 2s H 4s 3p 1d O 
        self[("TIP3P", "CAMB3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6633832,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3498616, ],
( 'O1' , 'quadrupole') : [ -3.4627832,  0.0000000,  -0.0000000,  -5.1253659,  -0.0000000,  -4.3172380, ],
( 'O1' , 'alpha' ) : [ 4.0771750,  -0.0000000,  0.0000000,  4.7333086,  0.0000000,  3.9781079, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -2.7814435,  -0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -0.2154837,  -0.0000000,  1.2452045, ],
( 'H2' , 'charge') : 0.3316916,
( 'H2' , 'dipole') : [ -0.1645672,  -0.0000000,  -0.1133423, ],
( 'H2' , 'quadrupole') : [ -0.2000174,  0.0000000,  0.2134609,  -0.4509118,  -0.0000000,  -0.2840164, ],
( 'H2' , 'alpha' ) : [ 1.5280337,  -0.0000000,  1.1820103,  0.8833836,  0.0000000,  1.3544736, ],
( 'H2' , 'beta'  ) : [ -4.3304122,  0.0000000,  -4.8046012,  -1.1748903,  -0.0000000,  -4.5806438,  0.0000000,  -0.3550927,  -0.0000000,  -3.2433141, ],
( 'H3' , 'charge') : 0.3316916,
( 'H3' , 'dipole') : [ 0.1645672,  -0.0000000,  -0.1133423, ],
( 'H3' , 'quadrupole') : [ -0.2000174,  -0.0000000,  -0.2134609,  -0.4509118,  0.0000000,  -0.2840164, ],
( 'H3' , 'alpha' ) : [ 1.5280337,  -0.0000000,  -1.1820103,  0.8833836,  0.0000000,  1.3544736, ],
( 'H3' , 'beta'  ) : [ 4.3304122,  -0.0000000,  -4.8046012,  1.1748903,  0.0000000,  4.5806438,  -0.0000000,  -0.3550927,  -0.0000000,  -3.2433141, ],
}
        self[("TIP3P", "CAMB3LYP", "ANO631", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8576464, ],
( 'X', 'quadrupole') : [ -3.8628180,  0.0000000,  0.0000000,  -6.0271895,  0.0000000,  -4.8852708, ],
( 'X', 'alpha' ) : [ 7.1332424,  0.0000000,  0.0000000,  6.5000758,  0.0000000,  6.6870551, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.3906459,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -0.9256691,  0.0000000,  -5.2414237, ],
}



#CAMB3LYP ANO631PGS 3s 1p H 4s 3p 1d O 
        self[("TIP3P", "CAMB3LYP", "ANO631PGS", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6846588,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3019081, ],
( 'O1' , 'quadrupole') : [ -3.6055380,  0.0000000,  0.0000000,  -5.0523459,  -0.0000000,  -4.3411008, ],
( 'O1' , 'alpha' ) : [ 4.4191711,  -0.0000000,  0.0000000,  5.0858773,  0.0000000,  4.2616221, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -2.9504130,  0.0000000,  0.0000000,  -0.0000000,  -0.0000000,  0.4000762,  -0.0000000,  3.5937864, ],
( 'H2' , 'charge') : 0.3423294,
( 'H2' , 'dipole') : [ -0.1581274,  -0.0000000,  -0.1369731, ],
( 'H2' , 'quadrupole') : [ -0.1126159,  0.0000000,  0.2654323,  -0.4310872,  0.0000000,  -0.2143753, ],
( 'H2' , 'alpha' ) : [ 1.9912731,  0.0000000,  1.1914290,  1.3140868,  0.0000000,  1.7300816, ],
( 'H2' , 'beta'  ) : [ -7.3025452,  0.0000000,  -4.5435406,  -3.3605382,  -0.0000000,  -5.0981115,  0.0000000,  -2.0196392,  -0.0000000,  -5.2249357, ],
( 'H3' , 'charge') : 0.3423294,
( 'H3' , 'dipole') : [ 0.1581274,  -0.0000000,  -0.1369731, ],
( 'H3' , 'quadrupole') : [ -0.1126159,  -0.0000000,  -0.2654323,  -0.4310872,  0.0000000,  -0.2143753, ],
( 'H3' , 'alpha' ) : [ 1.9912731,  -0.0000000,  -1.1914290,  1.3140868,  0.0000000,  1.7300816, ],
( 'H3' , 'beta'  ) : [ 7.3025452,  -0.0000000,  -4.5435406,  3.3605382,  0.0000000,  5.0981115,  0.0000000,  -2.0196392,  -0.0000000,  -5.2249357, ],
}
        self[("TIP3P", "CAMB3LYP", "ANO631PGS", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7859867, ],
( 'X', 'quadrupole') : [ -3.8307698,  0.0000000,  0.0000000,  -5.9145203,  0.0000000,  -4.7698514, ],
( 'X', 'alpha' ) : [ 8.4017173,  0.0000000,  0.0000000,  7.7140509,  0.0000000,  7.7217853, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.0374942,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -3.6392022,  0.0000000,  -6.8560850, ],
}

#ANOPVTZ 3s 2p 1d  // 4s 3p 2d 1f
        self[("TIP3P", "CAMB3LYP", "ANOPVTZ", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6844446,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2611940, ],
( 'O1' , 'quadrupole') : [ -3.6211613,  0.0000000,  0.0000000,  -5.0431442,  -0.0000000,  -4.2827265, ],
( 'O1' , 'alpha' ) : [ 5.5258354,  -0.0000000,  0.0000000,  6.4505478,  0.0000000,  6.0293427, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -3.1372136,  -0.0000000,  0.0000000,  0.0000000,  0.0000000,  1.3749003,  -0.0000000,  7.3890158, ],
( 'H2' , 'charge') : 0.3422223,
( 'H2' , 'dipole') : [ -0.1632221,  -0.0000000,  -0.1376622, ],
( 'H2' , 'quadrupole') : [ -0.1185328,  0.0000000,  0.2413706,  -0.4298241,  0.0000000,  -0.2610745, ],
( 'H2' , 'alpha' ) : [ 2.2154390,  0.0000000,  1.1998646,  1.5014286,  0.0000000,  1.8565570, ],
( 'H2' , 'beta'  ) : [ -11.5317582,  0.0000000,  -5.3355330,  -5.2148568,  0.0000000,  -5.8292958,  -0.0000000,  -2.1539414,  -0.0000000,  -7.1269990, ],
( 'H3' , 'charge') : 0.3422223,
( 'H3' , 'dipole') : [ 0.1632221,  -0.0000000,  -0.1376622, ],
( 'H3' , 'quadrupole') : [ -0.1185328,  -0.0000000,  -0.2413706,  -0.4298241,  0.0000000,  -0.2610745, ],
( 'H3' , 'alpha' ) : [ 2.2154390,  -0.0000000,  -1.1998646,  1.5014286,  0.0000000,  1.8565570, ],
( 'H3' , 'beta'  ) : [ 11.5317582,  -0.0000000,  -5.3355330,  5.2148568,  0.0000000,  5.8292958,  -0.0000000,  -2.1539414,  -0.0000000,  -7.1269990, ],

}
        self[("TIP3P", "CAMB3LYP", "ANOPVTZ", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7436573, ],
( 'X', 'quadrupole') : [ -3.8582269,  0.0000000,  0.0000000,  -5.9027924,  0.0000000,  -4.8048755, ],
( 'X', 'alpha' ) : [ 9.9567134,  0.0000000,  0.0000000,  9.4534050,  0.0000000,  9.7424567, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.8082796,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -2.9329825,  0.0000000,  -6.8649822, ],
}

#ANOTHIRD 3s 2p 1d  // 5s 4p 3d 2f

        self[("TIP3P", "CAMB3LYP", "ANOTHIRD", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6863533,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.2588157, ],
( 'O1' , 'quadrupole') : [ -3.5974994,  0.0000000,  -0.0000000,  -4.9044294,  -0.0000000,  -4.2239772, ],
( 'O1' , 'alpha' ) : [ 5.5817250,  -0.0000000,  -0.0000000,  6.4307813,  0.0000000,  6.0279682, ],
( 'O1' , 'beta'  ) : [ 0.0000000,  -0.0000000,  -1.7588954,  -0.0000000,  0.0000000,  -0.0000000,  0.0000000,  1.0903561,  -0.0000000,  6.2252282, ],
( 'H2' , 'charge') : 0.3431767,
( 'H2' , 'dipole') : [ -0.1633140,  -0.0000000,  -0.1382209, ],
( 'H2' , 'quadrupole') : [ -0.1113055,  0.0000000,  0.2551677,  -0.4512720,  -0.0000000,  -0.2539449, ],
( 'H2' , 'alpha' ) : [ 2.1658327,  0.0000000,  1.1375276,  1.4202005,  0.0000000,  1.7533974, ],
( 'H2' , 'beta'  ) : [ -11.1608447,  0.0000000,  -5.0725630,  -3.6620041,  -0.0000000,  -5.2976277,  0.0000000,  -1.5437095,  0.0000000,  -6.7430715, ],
( 'H3' , 'charge') : 0.3431767,
( 'H3' , 'dipole') : [ 0.1633140,  -0.0000000,  -0.1382209, ],
( 'H3' , 'quadrupole') : [ -0.1113055,  -0.0000000,  -0.2551677,  -0.4512720,  0.0000000,  -0.2539449, ],
( 'H3' , 'alpha' ) : [ 2.1658327,  -0.0000000,  -1.1375276,  1.4202005,  0.0000000,  1.7533974, ],
( 'H3' , 'beta'  ) : [ 11.1608447,  -0.0000000,  -5.0725630,  3.6620041,  0.0000000,  5.2976277,  -0.0000000,  -1.5437095,  -0.0000000,  -6.7430715, ],
}
        self[("TIP3P", "CAMB3LYP", "ANOTHIRD", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7422749, ],
( 'X', 'quadrupole') : [ -3.8201104,  0.0000000,  0.0000000,  -5.8069734,  0.0000000,  -4.7318670, ],
( 'X', 'alpha' ) : [ 9.9133904,  0.0000000,  0.0000000,  9.2711823,  0.0000000,  9.5347630, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -11.9040214,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -1.9970629,  0.0000000,  -7.2609148, ],
}

#ANOMAX 5s 4p 3d  // 5s 4p 3d 2f
        self[("TIP3P", "CAMB3LYP", "ANOMAX", True, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'O1' , 'charge') : -0.6855003,
( 'O1' , 'dipole') : [ -0.0000000,  0.0000000,  0.2727117, ],
( 'O1' , 'quadrupole') : [ -3.6005878,  -0.0000000,  0.0000000,  -4.9162553,  0.0000000,  -4.1977819, ],
( 'O1' , 'alpha' ) : [ 5.5724040,  0.0000000,  -0.0000000,  6.2052856,  0.0000000,  5.7572820, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  0.0000000,  -3.4025122,  -0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.1752134,  -0.0000000,  5.6084778, ],
( 'H2' , 'charge') : 0.3427501,
( 'H2' , 'dipole') : [ -0.1565261,  -0.0000000,  -0.1443252, ],
( 'H2' , 'quadrupole') : [ -0.1244614,  0.0000000,  0.2572952,  -0.4430247,  -0.0000000,  -0.2513335, ],
( 'H2' , 'alpha' ) : [ 2.2483627,  -0.0000000,  1.1422370,  1.5797225,  -0.0000000,  1.9462719, ],
( 'H2' , 'beta'  ) : [ -11.5280869,  -0.0000000,  -4.3507912,  -4.1920299,  -0.0000000,  -4.7024329,  -0.0000000,  -1.8178168,  0.0000000,  -6.5348107, ],
( 'H3' , 'charge') : 0.3427501,
( 'H3' , 'dipole') : [ 0.1565261,  -0.0000000,  -0.1443252, ],
( 'H3' , 'quadrupole') : [ -0.1244614,  0.0000000,  -0.2572952,  -0.4430247,  0.0000000,  -0.2513335, ],
( 'H3' , 'alpha' ) : [ 2.2483627,  -0.0000000,  -1.1422370,  1.5797225,  -0.0000000,  1.9462719, ],
( 'H3' , 'beta'  ) : [ 11.5280869,  -0.0000000,  -4.3507912,  4.1920299,  0.0000000,  4.7024329,  -0.0000000,  -1.8178168,  -0.0000000,  -6.5348107, ],
}

        self[("TIP3P", "CAMB3LYP", "ANOMAX", False, "0.0")] = \
{
'meta' : { 'label' : 'element_str_order', },
( 'X', 'charge') : -0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.7430177, ],
( 'X', 'quadrupole') : [ -3.8495106,  0.0000000,  0.0000000,  -5.8023047,  0.0000000,  -4.7004489, ],
( 'X', 'alpha' ) : [ 10.0691294,  0.0000000,  0.0000000,  9.3647306,  0.0000000,  9.6498258, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -12.1040946,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -3.4604202,  0.0000000,  -7.4611436, ],
}
#




#
###########
#       Ano-1 basis set 3s 2p on H / 4s3p2d on Oxygen
##########
        self[("TIP3P", "HF", "ANOAUG", True, "0.0")] = \
        {
( 'O1', "charge") :  -0.68786 ,
( 'H2', "charge") :  0.34393 ,
( 'H3', "charge") :  0.34393 ,
( 'O1', "dipole") : [ 0.00000, 0.00000, 0.26027 ],
( 'H2', "dipole") : [ -0.14810, 0.00000, -0.12319 ],
( 'H3', "dipole") : [ 0.14810, 0.00000, -0.12319 ],
( 'O1', "quadrupole") : [ -3.48423, -0.00000, 0.00000, -4.89434, 0.00000, -4.14731 ],
( 'H2', "quadrupole") : [ -0.12340, -0.00000, 0.22242, -0.39891, -0.00000, -0.23252 ],
( 'H3', "quadrupole") : [ -0.12340, -0.00000, -0.22242, -0.39891, 0.00000, -0.23252 ],
( 'O1', "alpha") : [ 4.99392, 0.00000, -0.00000, 5.58422, -0.00000, 5.20886 ],
( 'H2', "alpha") : [ 2.06292, -0.00000, 1.15705, 1.11523, -0.00000, 1.60390 ],
( 'H3', "alpha") : [ 2.06292, 0.00000, -1.15705, 1.11523, -0.00000, 1.60390 ],
( 'O1', "beta") : [ 0.00000, 0.00000, -2.84514, -0.00000, -0.00000, 0.00000, 0.00000, 2.58100, 0.00000, 6.82314 ],
( 'H2', "beta") : [ -10.21889, 0.00000, -4.86332, -3.04758, 0.00000, -4.74382, -0.00000, -1.19046, -0.00000, -6.00406 ],
( 'H3', "beta") : [ 10.21889, 0.00000, -4.86332, 3.04758, -0.00000, 4.74382, -0.00000, -1.19046, 0.00000, -6.00406 ],
}

        self[("TIP3P", "HF", "ANOAUG", False, "0.0")] = \
        {
( 'O1', "charge") :  -0.00000 ,
( 'O1', "dipole") : [ 0.00000, 0.00000, 0.77545 ],
( 'O1', "quadrupole") : [ -3.73103, -0.00000, 0.00000, -5.69216, 0.00000, -4.61235 ],
( 'O1', "alpha") : [ 9.11976, 0.00000, -0.00000, 7.81467, -0.00000, 8.41665 ],
( 'O1', "beta") : [ 0.00000, 0.00000, -12.57178, 0.00000, -0.00000, 0.00000, 0.00000, 0.20008, 0.00000, -5.18499 ],
}
###########
#       Ano-1 basis set 3s 2p on H / 4s3p2d on Oxygen But with Camb3lyp
##########
        self[("TIP3P", "CAMB3LYP", "ANOAUG", True, "0.0")] = \
        {
( 'O1', "charge") :  -0.683 ,
( 'H2', "charge") :  0.341 ,
( 'H3', "charge") :  0.341 ,
( 'O1', "dipole") : [ -0.000, 0.000, 0.254 ],
( 'H2', "dipole") : [ -0.162, 0.000, -0.132 ],
( 'H3', "dipole") : [ 0.162, 0.000, -0.132 ],
( 'O1', "quadrupole") : [ -3.587, -0.000, -0.000, -5.106, 0.000, -4.300 ],
( 'H2', "quadrupole") : [ -0.133, -0.000, 0.218, -0.414, -0.000, -0.258 ],
( 'H3', "quadrupole") : [ -0.133, -0.000, -0.218, -0.414, 0.000, -0.258 ],
( 'O1', "alpha") : [ 5.518, 0.000, -0.000, 6.618, -0.000, 6.020 ],
( 'H2', "alpha") : [ 2.206, -0.000, 1.252, 1.423, -0.000, 1.850 ],
( 'H3', "alpha") : [ 2.206, 0.000, -1.252, 1.423, -0.000, 1.850 ],
( 'O1', "beta") : [ -0.000, 0.000, -3.243, -0.000, -0.000, -0.000, 0.000, 1.812, 0.000, 7.529 ],
( 'H2', "beta") : [ -11.391, 0.000, -5.560, -5.157, 0.000, -5.897, -0.000, -2.108, -0.000, -7.278 ],
( 'H3', "beta") : [ 11.391, 0.000, -5.560, 5.157, -0.000, 5.897, -0.000, -2.108, 0.000, -7.278 ],
}

###########
#


        self[("TIP3P", "HF", "ANOPVDZ", True, "0.0238927")] = \
{
("O1","charge"): -0.678,
("H2","charge"):  0.339,
("H3","charge"):  0.339,

("O1", "dipole"): [  0.000,    -0.000,     0.298, ],
("H2", "dipole"): [ -0.154,     0.000,    -0.131, ],
("H3", "dipole"): [  0.154,     0.000,    -0.131, ],

("O1", "alpha"): [ 3.843, -0.000, -0.000,  3.245, -0.000,  3.710,],
("H2", "alpha"): [ 1.838, -0.000,  1.091,  1.254, -0.000,  1.580,],
("H3", "alpha"): [ 1.838, -0.000, -1.091,  1.254,  0.000,  1.580,],

("O1", "beta") :  [0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta") :  [0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta") :  [0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
}

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.0238927")] = \
                {
("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ 0.00000,  0.00000,  0.78706,   ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [7.51884, -0.00000,  -0.00000, 5.75300,  -0.00000,  6.87101,   ],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],

("O1", "beta") : [0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta") : [0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta") : [0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
                        }

# At 1064 nm

        self[("TIP3P", "HF", "ANOPVDZ", True, "0.0428227")] = \
{
("O1","charge"):  -0.678,
("H2","charge"):  0.339, 
("H3","charge"):  0.339, 

("O1", "dipole"): [  0.000,    -0.000,     0.298, ], 
("H2", "dipole"): [ -0.154,     0.000,    -0.131, ],
("H3", "dipole"): [  0.154,     0.000,    -0.131, ],

("O1", "alpha"): [ 3.825, -0.000,  -0.000, 3.172, -0.000,  3.667,],
("H2", "alpha"): [ 1.856, -0.000,   1.112, 1.305, -0.000,  1.613,],
("H3", "alpha"): [ 1.856, -0.000,  -1.112, 1.305,  0.000,  1.613,],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],

        }

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.0428227")] = \
                {

("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ 0.00000,  0.00000,  0.78706,   ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 7.53651, -0.00000,  -0.00000, 5.78202,  -0.00000,  6.89360,   ],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
                }

# At 589 nm
        self[("TIP3P", "HF", "ANOPVDZ", True, "0.0773571")] = \
                {
("O1","charge"): -0.678,
("H2","charge"): 0.339, 
("H3","charge"): 0.339, 

("O1", "dipole"): [  0.000,    -0.000,     0.298, ], 
("H2", "dipole"): [ -0.154,     0.000,    -0.131, ],
("H3", "dipole"): [  0.154,     0.000,    -0.131, ],

("O1", "alpha"): [ 3.801, -0.000,  0.000,  3.045, -0.000,  3.599,],
("H2", "alpha"): [ 1.897, -0.000,  1.157,  1.419, -0.000,  1.686,],
("H3", "alpha"): [ 1.897, -0.000, -1.157,  1.419,  0.000,  1.686,],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
        }

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.0773571")] = \
                {

("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ 0.00000,  0.00000,  0.78706,   ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 7.59551, -0.00000,  0.00000, 5.88201, -0.00000,  6.96982,],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],

}

##############################
## Properties for demonstrating resonance with cluster of 10 molecules, only DIST
##############################

        self[("TIP3P", "HF", "ANOPVDZ", True, "0.1")] = \
                {

("O1","charge"):  -0.67786 ,
("H2","charge"):  0.33893 ,
("H3","charge"):  0.33893 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.29809 ],
("H2", "dipole"): [ -0.15448, -0.00000, -0.13077 ],
("H3", "dipole"): [ 0.15448, -0.00000, -0.13077 ],

("O1", "alpha"): [ 3.79335, -0.00000, 0.00000, 2.96308, 0.00000, 3.56142 ],
("H2", "alpha"): [ 1.93033, 0.00000, 1.19292, 1.51153, 0.00000, 1.74261 ],
("H3", "alpha"): [ 1.93033, -0.00000, -1.19292, 1.51153, 0.00000, 1.74261 ],
}

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.1")] = \
                {
("O1","charge"):  0.0 ,
("H2","charge"):  0.0 ,
("H3","charge"):  0.0 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.78706 ],
("H2", "dipole"): [ 0.00000, -0.00000, 0.0     ],
("H3", "dipole"): [ 0.00000, -0.00000, 0.0     ],

("O1", "alpha"): [ 7.65401, -0.00000, -0.00000, 5.98613, 0.00000, 7.04663 ],
("H2", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
("H3", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
}




        self[("TIP3P", "HF", "ANOPVDZ", True, "0.2")] = \
                {
("O1","charge"):  -0.67786 ,
("H2","charge"):  0.33893 ,
("H3","charge"):  0.33893 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.29809 ],
("H2", "dipole"): [ -0.15448, -0.00000, -0.13077 ],
("H3", "dipole"): [ 0.15448, -0.00000, -0.13077 ],

("O1", "alpha"): [ 3.83278, -0.00000, -0.00000, 2.50658, -0.00000, 3.45435 ],
("H2", "alpha"): [ 2.15396, -0.00000, 1.43327, 2.30647, 0.00000, 2.14337 ],
("H3", "alpha"): [ 2.15396, -0.00000, -1.43327, 2.30647, 0.00000, 2.14337 ],
}


        self[("TIP3P", "HF", "ANOPVDZ", False, "0.2")] = \
                {
("O1","charge"):  0.0 ,
("H2","charge"):  0.0 ,
("H3","charge"):  0.0 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.78706 ],
("H2", "dipole"): [ 0.00000, -0.00000, 0.0     ],
("H3", "dipole"): [ 0.00000, -0.00000, 0.0     ],

("O1", "alpha"): [ 8.14070, -0.00000, -0.00000, 7.11951, 0.00000, 7.74108 ],
("H2", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
("H3", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
}



        self[("TIP3P", "HF", "ANOPVDZ", True, "0.25")] = \
                {
("O1","charge"):  -0.67786 ,
("H2","charge"):  0.33893 ,
("H3","charge"):  0.33893 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.29809 ],
("H2", "dipole"): [ -0.15448, -0.00000, -0.13077 ],
("H3", "dipole"): [ 0.15448, -0.00000, -0.13077 ],

("O1", "alpha"): [ 3.90757, -0.00000, -0.00000, 1.99481, 0.00000, 3.42902 ],
("H2", "alpha"): [ 2.33518, -0.00000, 1.64022, 3.44417, -0.00000, 2.52047 ],
("H3", "alpha"): [ 2.33518, -0.00000, -1.64022, 3.44417, -0.00000, 2.52047 ],
}

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.25")] = \
                {
("O1","charge"):  0.0 ,
("H2","charge"):  0.0 ,
("H3","charge"):  0.0 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.78706 ],
("H2", "dipole"): [ 0.00000, -0.00000, 0.0     ],
("H3", "dipole"): [ 0.00000, -0.00000, 0.0     ],

("O1", "alpha"): [ 8.57793, -0.00000, -0.00000, 8.88315, -0.00000, 8.46996 ],
("H2", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
("H3", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
}




        self[("TIP3P", "HF", "ANOPVDZ", True, "0.3")] = \
                {
("O1","charge"):  -0.67786 ,
("H2","charge"):  0.33893 ,
("H3","charge"):  0.33893 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.29809 ],
("H2", "dipole"): [ -0.15448, -0.00000, -0.13077 ],
("H3", "dipole"): [ 0.15448, -0.00000, -0.13077 ],

("O1", "alpha"): [ 4.03526, 0.00000, -0.00000, -0.15844, 0.00000, 3.41271 ],
("H2", "alpha"): [ 2.59806, 0.00000, 1.97668, 8.42570, -0.00001, 3.20745 ],
("H3", "alpha"): [ 2.59806, 0.00000, -1.97668, 8.42570, -0.00001, 3.20745 ],
}

        self[("TIP3P", "HF", "ANOPVDZ", False, "0.3")] = \
                {
("O1","charge"):  0.0 ,
("H2","charge"):  0.0 ,
("H3","charge"):  0.0 ,

("O1", "dipole"): [ 0.00000, -0.00000, 0.78706 ],
("H2", "dipole"): [ 0.00000, -0.00000, 0.0     ],
("H3", "dipole"): [ 0.00000, -0.00000, 0.0     ],

("O1", "alpha"): [ 9.23138, -0.00000, -0.00000, 16.69294, 0.00000, 9.82760],
("H2", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
("H3", "alpha"): [ 0., 0., 0., 0., 0., 0.,], 
}
###########
#       Ano-1 basis set 3s2p1d contr on H / 5s4p3d2f contr on O
#       HF functional, used to test larger basis of our point dipole model
##########


###########
#       Basis corresponding to 6-31+G* contraction 2s (H) / 4s3p1d (O)
#       Both for HF and B3LYP At different perturbing frequencies
                        
# HF PART
        self[("TIP3P_PDB", "HF", "ANO631", True, "0.0")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66388,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34510, ],
( 'OW' , 'quadrupole') : [ -3.34420,  0.00000,  -0.00000,  -4.87379,  -0.00000,  -4.12578, ],
( 'OW' , 'alpha' ) : [ 3.78327,  -0.00000,  0.00000,  3.96610,  0.00000,  3.52668, ],
( 'OW' , 'beta'  ) : [ -0.00000,  -0.00000,  -2.98430,  0.00000,  -0.00000,  0.00000,  -0.00000,  1.26745,  -0.00000,  2.16731, ],
( 'HW1', 'charge') : 0.33194,
( 'HW1', 'dipole') : [ -0.16058,  -0.00000,  -0.11299, ],
( 'HW1', 'quadrupole') : [ -0.19022,  0.00000,  0.21775,  -0.44564,  0.00000,  -0.27199, ],
( 'HW1', 'alpha' ) : [ 1.55235,  -0.00000,  1.15495,  0.60860,  0.00000,  1.21104, ],
( 'HW1', 'beta'  ) : [ -4.46820,  0.00000,  -4.55909,  -0.05602,  0.00000,  -3.72030,  -0.00000,  0.46040,  0.00000,  -2.40410, ],
( 'HW2', 'charge') : 0.33194,
( 'HW2', 'dipole') : [ 0.16058,  -0.00000,  -0.11299, ],
( 'HW2', 'quadrupole') : [ -0.19022,  -0.00000,  -0.21775,  -0.44564,  0.00000,  -0.27199, ],
( 'HW2', 'alpha' ) : [ 1.55235,  -0.00000,  -1.15495,  0.60860,  0.00000,  1.21104, ],
( 'HW2', 'beta'  ) : [ 4.46820,  -0.00000,  -4.55909,  0.05602,  0.00000,  3.72030,  -0.00000,  0.46040,  -0.00000,  -2.40410, ],
}

        self[("TIP3P_PDB", "HF", "ANO631", False, "0.0")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge'  ) : 0.00000,
( 'X', 'dipole'  ) : [ 0.00000,  0.00000,  0.85414, ],
( 'X', 'quadrupole') : [ -3.72464,  0.00000,  0.00000,  -5.76507,  0.00000,  -4.66976, ],
( 'X', 'alpha'   ) : [ 6.88797,  0.00000,  0.00000,  5.18330,  0.00000,  5.94876, ],
( 'X', 'beta'    ) : [ 0.00000,  0.00000,  -12.10248,  0.00000,  0.00000,  0.00000,  0.00000,  2.18825,  0.00000,  -2.64089, ],
}

#700 nm
        self[("TIP3P_PDB", "HF", "ANO631", True, "0.0650905")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6638768,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3450971, ],
( 'OW' , 'quadrupole') : [ -3.3441982,  0.0000000,  -0.0000000,  -4.8737884,  -0.0000000,  -4.1257814, ],
( 'OW' , 'alpha' ) : [ 3.7582101,  -0.0000000,  0.0000000,  3.7822586,  0.0000000,  3.4214895, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3319384,
( 'HW1', 'dipole') : [ -0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW1', 'quadrupole') : [ -0.1902230,  0.0000000,  0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW1', 'alpha' ) : [ 1.5954217,  -0.0000000,  1.2139515,  0.7462187,  0.0000000,  1.3003492, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3319384,
( 'HW2', 'dipole') : [ 0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW2', 'quadrupole') : [ -0.1902230,  -0.0000000,  -0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW2', 'alpha' ) : [ 1.5954217,  -0.0000000,  -1.2139515,  0.7462187,  0.0000000,  1.3003492, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}
#600 nm
        self[("TIP3P_PDB", "HF", "ANO631", True, "0.0759389")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6638768,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3450971, ],
( 'OW' , 'quadrupole') : [ -3.3441982,  0.0000000,  -0.0000000,  -4.8737884,  -0.0000000,  -4.1257814, ],
( 'OW' , 'alpha' ) : [ 3.7592323,  -0.0000000,  0.0000000,  3.7570477,  0.0000000,  3.4093329, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3319384,
( 'HW1', 'dipole') : [ -0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW1', 'quadrupole') : [ -0.1902230,  0.0000000,  0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW1', 'alpha' ) : [ 1.6061347,  -0.0000000,  1.2273446,  0.7761139,  0.0000000,  1.3200656, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3319384,
( 'HW2', 'dipole') : [ 0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW2', 'quadrupole') : [ -0.1902230,  -0.0000000,  -0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW2', 'alpha' ) : [ 1.6061347,  -0.0000000,  -1.2273446,  0.7761139,  0.0000000,  1.3200656, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

#500 nm
        self[("TIP3P_PDB", "HF", "ANO631", True, "0.0911267")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6638768,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3450971, ],
( 'OW' , 'quadrupole') : [ -3.3441982,  0.0000000,  -0.0000000,  -4.8737884,  -0.0000000,  -4.1257814, ],
( 'OW' , 'alpha' ) : [ 3.7632072,  -0.0000000,  0.0000000,  3.7233754,  0.0000000,  3.3946633, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3319384,
( 'HW1', 'dipole') : [ -0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW1', 'quadrupole') : [ -0.1902230,  0.0000000,  0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW1', 'alpha' ) : [ 1.6229973,  -0.0000000,  1.2480570,  0.8226198,  0.0000000,  1.3504860, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3319384,
( 'HW2', 'dipole') : [ 0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW2', 'quadrupole') : [ -0.1902230,  -0.0000000,  -0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW2', 'alpha' ) : [ 1.6229973,  -0.0000000,  -1.2480570,  0.8226198,  0.0000000,  1.3504860, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

#400 nm
        self[("TIP3P_PDB", "HF", "ANO631", True, "0.1139084")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6638768,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3450971, ],
( 'OW' , 'quadrupole') : [ -3.3441982,  0.0000000,  -0.0000000,  -4.8737884,  -0.0000000,  -4.1257814, ],
( 'OW' , 'alpha' ) : [ 3.7748778,  -0.0000000,  0.0000000,  3.6750871,  0.0000000,  3.3776254, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3319384,
( 'HW1', 'dipole') : [ -0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW1', 'quadrupole') : [ -0.1902230,  0.0000000,  0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW1', 'alpha' ) : [ 1.6526546,  -0.0000000,  1.2838412,  0.9049187,  0.0000000,  1.4031066, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3319384,
( 'HW2', 'dipole') : [ 0.1605791,  -0.0000000,  -0.1129931, ],
( 'HW2', 'quadrupole') : [ -0.1902230,  -0.0000000,  -0.2177550,  -0.4456375,  0.0000000,  -0.2719917, ],
( 'HW2', 'alpha' ) : [ 1.6526546,  -0.0000000,  -1.2838412,  0.9049187,  0.0000000,  1.4031066, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

                        
# For static for SPC, HF, ANO631, LoProp only
        self[("SPC", "HF", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6602930,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3377100, ],
( 'OW' , 'quadrupole') : [ -3.3290686,  0.0000000,  0.0000000,  -4.9267521,  -0.0000000,  -4.2405456, ],
( 'OW' , 'alpha' ) : [ 4.1989134,  -0.0000000,  0.0000000,  4.0276319,  0.0000000,  3.7572878, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3301465,
( 'HW1', 'dipole') : [ -0.1492843,  0.0000000,  -0.1098534, ],
( 'HW1', 'quadrupole') : [ -0.1773552,  -0.0000000,  0.2299409,  -0.4599220,  -0.0000000,  -0.2863321, ],
( 'HW1', 'alpha' ) : [ 1.8515829,  -0.0000000,  1.2853970,  0.6098903,  -0.0000000,  1.2790333, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3301465,
( 'HW2', 'dipole') : [ 0.1492843,  -0.0000000,  -0.1098534, ],
( 'HW2', 'quadrupole') : [ -0.1773552,  -0.0000000,  -0.2299409,  -0.4599220,  0.0000000,  -0.2863321, ],
( 'HW2', 'alpha' ) : [ 1.8515829,  -0.0000000,  -1.2853970,  0.6098903,  0.0000000,  1.2790333, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

                        
# From static to 400 nm for SPC, B3LYP, ANO631, LoProp only
        self[("SPC", "B3LYP", "ANO631", True, "0.0")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6569773,
( 'OW' , 'dipole') : [ -0.0000000,  -0.0000000,  0.3344862, ],
( 'OW' , 'quadrupole') : [ -3.4572033,  -0.0000000,  0.0000000,  -5.1851663,  -0.0000000,  -4.4356938, ],
( 'OW' , 'alpha' ) : [ 4.5238575,  0.0000000,  -0.0000000,  4.8837172,  0.0000000,  4.2809177, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3284886,
( 'HW1', 'dipole') : [ -0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW1', 'quadrupole') : [ -0.1818431,  -0.0000000,  0.2291927,  -0.4614100,  -0.0000000,  -0.2905138, ],
( 'HW1', 'alpha' ) : [ 1.8192606,  0.0000000,  1.3278142,  0.9096664,  0.0000000,  1.4492244, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3284886,
( 'HW2', 'dipole') : [ 0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW2', 'quadrupole') : [ -0.1818431,  -0.0000000,  -0.2291927,  -0.4614100,  0.0000000,  -0.2905138, ],
( 'HW2', 'alpha' ) : [ 1.8192606,  0.0000000,  -1.3278142,  0.9096664,  0.0000000,  1.4492244, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

#700 nm
        self[("SPC", "B3LYP", "ANO631", True, "0.0650905")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6569773,
( 'OW' , 'dipole') : [ -0.0000000,  -0.0000000,  0.3344862, ],
( 'OW' , 'quadrupole') : [ -3.4572033,  -0.0000000,  0.0000000,  -5.1851663,  -0.0000000,  -4.4356938, ],
( 'OW' , 'alpha' ) : [ 4.4914581,  0.0000000,  -0.0000000,  4.5122495,  0.0000000,  4.0758544, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3284886,
( 'HW1', 'dipole') : [ -0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW1', 'quadrupole') : [ -0.1818431,  -0.0000000,  0.2291927,  -0.4614100,  -0.0000000,  -0.2905138, ],
( 'HW1', 'alpha' ) : [ 1.8832803,  0.0000000,  1.4284145,  1.2044982,  0.0000000,  1.6274639, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3284886,
( 'HW2', 'dipole') : [ 0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW2', 'quadrupole') : [ -0.1818431,  -0.0000000,  -0.2291927,  -0.4614100,  0.0000000,  -0.2905138, ],
( 'HW2', 'alpha' ) : [ 1.8832803,  0.0000000,  -1.4284145,  1.2044982,  0.0000000,  1.6274639, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}
#600 nm
        self[("SPC", "B3LYP", "ANO631", True, "0.0759389")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6569773,
( 'OW' , 'dipole') : [ -0.0000000,  -0.0000000,  0.3344862, ],
( 'OW' , 'quadrupole') : [ -3.4572033,  -0.0000000,  0.0000000,  -5.1851663,  -0.0000000,  -4.4356938, ],
( 'OW' , 'alpha' ) : [ 4.4946615,  0.0000000,  0.0000000,  4.4578332,  0.0000000,  4.0520629, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3284886,
( 'HW1', 'dipole') : [ -0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW1', 'quadrupole') : [ -0.1818431,  -0.0000000,  0.2291927,  -0.4614100,  -0.0000000,  -0.2905138, ],
( 'HW1', 'alpha' ) : [ 1.8993604,  0.0000000,  1.4516590,  1.2745131,  0.0000000,  1.6680541, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3284886,
( 'HW2', 'dipole') : [ 0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW2', 'quadrupole') : [ -0.1818431,  -0.0000000,  -0.2291927,  -0.4614100,  0.0000000,  -0.2905138, ],
( 'HW2', 'alpha' ) : [ 1.8993604,  0.0000000,  -1.4516590,  1.2745131,  0.0000000,  1.6680541, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],

}
#500 nm
        self[("SPC", "B3LYP", "ANO631", True, "0.0911267")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6569773,
( 'OW' , 'dipole') : [ -0.0000000,  -0.0000000,  0.3344862, ],
( 'OW' , 'quadrupole') : [ -3.4572033,  -0.0000000,  0.0000000,  -5.1851663,  -0.0000000,  -4.4356938, ],
( 'OW' , 'alpha' ) : [ 4.5034062,  0.0000000,  -0.0000000,  4.3813828,  0.0000000,  4.0230711, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3284886,
( 'HW1', 'dipole') : [ -0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW1', 'quadrupole') : [ -0.1818431,  -0.0000000,  0.2291927,  -0.4614100,  -0.0000000,  -0.2905138, ],
( 'HW1', 'alpha' ) : [ 1.9248162,  0.0000000,  1.4880407,  1.3883049,  0.0000000,  1.7318534, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3284886,
( 'HW2', 'dipole') : [ 0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW2', 'quadrupole') : [ -0.1818431,  -0.0000000,  -0.2291927,  -0.4614100,  0.0000000,  -0.2905138, ],
( 'HW2', 'alpha' ) : [ 1.9248162,  0.0000000,  -1.4880407,  1.3883049,  0.0000000,  1.7318534, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],

}
#400 nm
        self[("SPC", "B3LYP", "ANO631", True, "0.1139084")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6569773,
( 'OW' , 'dipole') : [ -0.0000000,  -0.0000000,  0.3344862, ],
( 'OW' , 'quadrupole') : [ -3.4572033,  -0.0000000,  0.0000000,  -5.1851663,  -0.0000000,  -4.4356938, ],
( 'OW' , 'alpha' ) : [ 4.5262021,  0.0000000,  -0.0000000,  4.2594896,  0.0000000,  3.9885194, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3284886,
( 'HW1', 'dipole') : [ -0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW1', 'quadrupole') : [ -0.1818431,  -0.0000000,  0.2291927,  -0.4614100,  -0.0000000,  -0.2905138, ],
( 'HW1', 'alpha' ) : [ 1.9700431,  0.0000000,  1.5523233,  1.6058018,  0.0000000,  1.8459897, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3284886,
( 'HW2', 'dipole') : [ 0.1560016,  -0.0000000,  -0.1141859, ],
( 'HW2', 'quadrupole') : [ -0.1818431,  -0.0000000,  -0.2291927,  -0.4614100,  0.0000000,  -0.2905138, ],
( 'HW2', 'alpha' ) : [ 1.9700431,  0.0000000,  -1.5523233,  1.6058018,  0.0000000,  1.8459897, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],

}


# CCSD with ano-1 4 3 1 on OXY 2s on H 
        self[("TIP3P_PDB", "CCSD", "ANO631", False, "0.0")] = \
                {
( 'X', "charge") :  0.0,
( 'X', "dipole") :  [ 0.,0.0, 0.853387490,], 
( 'X', "quadrupole") :  [ 0., 0., 0., 0., 0., 0.,], 
( 'X', "alpha") :  [ 7.02770140, 0.0, 0.0, 6.2860135, 0.0, 6.46317910 ],
( 'X', "beta") :  [ 0.0, 0.0, -12.189985, 0.0, 0.0, 0.0, 0.0, -0.75486706, 0.0, 
    -5.1183529, ],

}

# CAMB3LYP with ano-1 4 3 1 on OXY 2s on H For freqs static, 700, 600, 500, 400 nm
        self[("TIP3P_PDB", "CAMB3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6633800,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3498600, ],
( 'OW' , 'quadrupole') : [ -3.4627800,  0.0000000,  0.0000000,  -5.1253700,  -0.0000000,  -4.3172400, ],
( 'OW' , 'alpha' ) : [ 4.0771700,  -0.0000000,  0.0000000,  4.7333100,  0.0000000,  3.9781100, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3316900,
( 'HW1', 'dipole') : [ -0.1645700,  -0.0000000,  -0.1133400, ],
( 'HW1', 'quadrupole') : [ -0.2000200,  0.0000000,  0.2134600,  -0.4509100,  -0.0000000,  -0.2840200, ],
( 'HW1', 'alpha' ) : [ 1.5280300,  0.0000000,  1.1820100,  0.8833800,  0.0000000,  1.3544700, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3316900,
( 'HW2', 'dipole') : [ 0.1645700,  -0.0000000,  -0.1133400, ],
( 'HW2', 'quadrupole') : [ -0.2000200,  -0.0000000,  -0.2134600,  -0.4509100,  0.0000000,  -0.2840200, ],
( 'HW2', 'alpha' ) : [ 1.5280300,  -0.0000000,  -1.1820100,  0.8833800,  0.0000000,  1.3544700, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

# static point
        self[("TIP3P_PDB", "CAMB3LYP", "ANO631", False, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8576458, ],
( 'X', 'quadrupole') : [ -3.8628200,  0.0000000,  0.0000000,  -6.0271900,  0.0000000,  -4.8852800, ],
( 'X', 'alpha' ) : [ 7.1332300,  0.0000000,  0.0000000,  6.5000700,  0.0000000,  6.6870500, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

# 700 nm cam




# B3LYP with ano-1 4 3 1 on OXY 2s on H For freqs static, 1907, 1064, 589 nm

# static loprop

        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0")] = \
                {

'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6622867,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3427575, ],
( 'OW' , 'quadrupole') : [ -3.4641851,  0.0000000,  -0.0000000,  -5.1250616,  -0.0000000,  -4.3163864, ],
( 'OW' , 'alpha' ) : [ 4.1057421,  -0.0000000,  0.0000000,  4.7922852,  0.0000000,  4.0191241, ],
( 'OW' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -3.3316177,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -0.3221599,  -0.0000000,  0.7913706, ],
( 'HW1', 'charge') : 0.3311434,
( 'HW1', 'dipole') : [ -0.1661726,  -0.0000000,  -0.1162879, ],
( 'HW1', 'quadrupole') : [ -0.1963880,  0.0000000,  0.2158936,  -0.4475531,  -0.0000000,  -0.2780105, ],
( 'HW1', 'alpha' ) : [ 1.5380242,  -0.0000000,  1.1976512,  0.9066088,  -0.0000000,  1.3713771, ],
( 'HW1', 'beta'  ) : [ -4.5213728,  0.0000000,  -5.0806123,  -1.3549385,  0.0000000,  -4.8336536,  0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],
( 'HW2', 'charge') : 0.3311434,
( 'HW2', 'dipole') : [ 0.1661726,  -0.0000000,  -0.1162879, ],
( 'HW2', 'quadrupole') : [ -0.1963880,  -0.0000000,  -0.2158936,  -0.4475531,  0.0000000,  -0.2780105, ],
( 'HW2', 'alpha' ) : [ 1.5380242,  -0.0000000,  -1.1976512,  0.9066088,  0.0000000,  1.3713771, ],
( 'HW2', 'beta'  ) : [ 4.5213728,  -0.0000000,  -5.0806123,  1.3549385,  0.0000000,  4.8336536,  -0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],

( 'O1' , 'charge') : -0.6622867,
( 'O1' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3427575, ],
( 'O1' , 'quadrupole') : [ -3.4641851,  0.0000000,  -0.0000000,  -5.1250616,  -0.0000000,  -4.3163864, ],
( 'O1' , 'alpha' ) : [ 4.1057421,  -0.0000000,  0.0000000,  4.7922852,  0.0000000,  4.0191241, ],
( 'O1' , 'beta'  ) : [ -0.0000000,  -0.0000000,  -3.3316177,  0.0000000,  0.0000000,  0.0000000,  -0.0000000,  -0.3221599,  -0.0000000,  0.7913706, ],
( 'H2', 'charge') : 0.3311434,
( 'H2', 'dipole') : [ -0.1661726,  -0.0000000,  -0.1162879, ],
( 'H2', 'quadrupole') : [ -0.1963880,  0.0000000,  0.2158936,  -0.4475531,  -0.0000000,  -0.2780105, ],
( 'H2', 'alpha' ) : [ 1.5380242,  -0.0000000,  1.1976512,  0.9066088,  -0.0000000,  1.3713771, ],
( 'H2', 'beta'  ) : [ -4.5213728,  0.0000000,  -5.0806123,  -1.3549385,  0.0000000,  -4.8336536,  0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],
( 'H3', 'charge') : 0.3311434,
( 'H3', 'dipole') : [ 0.1661726,  -0.0000000,  -0.1162879, ],
( 'H3', 'quadrupole') : [ -0.1963880,  -0.0000000,  -0.2158936,  -0.4475531,  0.0000000,  -0.2780105, ],
( 'H3', 'alpha' ) : [ 1.5380242,  -0.0000000,  -1.1976512,  0.9066088,  0.0000000,  1.3713771, ],
( 'H3', 'beta'  ) : [ 4.5213728,  -0.0000000,  -5.0806123,  1.3549385,  0.0000000,  4.8336536,  -0.0000000,  -0.4631733,  -0.0000000,  -3.4792060, ],

}

# static point property

        self[("TIP3P_PDB", "B3LYP", "ANO631", False, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : 0.000000,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434372, ],
( 'X', 'quadrupole') : [ -3.8569611,  0.0000000,  0.0000000,  -6.0201678,  0.0000000,  -4.8724074, ],
( 'X', 'alpha' ) : [ 7.1817905,  0.0000000,  0.0000000,  6.6055028,  0.0000000,  6.7618783, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  -13.4928423,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  -1.2485065,  0.0000000,  -6.1670414, ],
}

# End of static TIP3P_PDB / B3LYP / ANO631, Non-loprop, static
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0238927")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.08507,  -0.00000,  0.00000,  4.64670,  0.00000,  3.94204, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.55327,  -0.00000,  1.22364,  0.99261,  -0.00000,  1.41809, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.55327,  -0.00000,  -1.22364,  0.99261,  0.00000,  1.41809, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0428227")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.07537,  -0.00000,  0.00000,  4.54430,  0.00000,  3.89070, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.56903,  -0.00000,  1.24854,  1.07374,  -0.00000,  1.46207, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.56903,  -0.00000,  -1.24854,  1.07374,  0.00000,  1.46207, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#589 nm
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0773571")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.07281,  -0.00000,  0.00000,  4.37531,  0.00000,  3.81640, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.60691,  -0.00000,  1.30529,  1.26397,  0.00000,  1.56190, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.60691,  -0.00000,  -1.30529,  1.26397,  0.00000,  1.56190, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#532 nm
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0856454")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.6622867,
( 'OW' , 'dipole') : [ 0.0000000,  -0.0000000,  0.3427575, ],
( 'OW' , 'quadrupole') : [ -3.4641851,  0.0000000,  0.0000000,  -5.1250616,  -0.0000000,  -4.3163864, ],
( 'OW' , 'alpha' ) : [ 4.0751415,  -0.0000000,  0.0000000,  4.3362619,  0.0000000,  3.8019538, ],
( 'OW' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW1', 'charge') : 0.3311434,
( 'HW1', 'dipole') : [ -0.1661726,  -0.0000000,  -0.1162879, ],
( 'HW1', 'quadrupole') : [ -0.1963880,  0.0000000,  0.2158936,  -0.4475531,  -0.0000000,  -0.2780105, ],
( 'HW1', 'alpha' ) : [ 1.6179039,  -0.0000000,  1.3214037,  1.3206175,  0.0000000,  1.5903876, ],
( 'HW1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HW2', 'charge') : 0.3311434,
( 'HW2', 'dipole') : [ 0.1661726,  -0.0000000,  -0.1162879, ],
( 'HW2', 'quadrupole') : [ -0.1963880,  -0.0000000,  -0.2158936,  -0.4475531,  0.0000000,  -0.2780105, ],
( 'HW2', 'alpha' ) : [ 1.6179039,  -0.0000000,  -1.3214037,  1.3206175,  0.0000000,  1.5903876, ],
( 'HW2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],

}

# 700
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0650905")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.07146,  -0.00000,  0.00000,  4.43364,  0.00000,  3.84011, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.59204,  -0.00000,  1.28330,  1.18883,  0.00000,  1.52318, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.59204,  -0.00000,  -1.28330,  1.18883,  0.00000,  1.52318, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#700 nm, Non loprop
        self[("TIP3P_PDB", "B3LYP", "ANO631", False, "0.0650905")] = \
                {
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : -0.000010,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434302, ],
( 'X', 'quadrupole') : [ -3.8569700,  0.0000000,  0.0000000,  -6.0201600,  0.0000000,  -4.8724100, ],
( 'X', 'alpha' ) : [ 7.2555400,  0.0000000,  0.0000000,  6.8113000,  0.0000000,  6.8864700, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

# 600
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0759389")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.07252,  -0.00000,  -0.00000,  4.38200,  0.00000,  3.81899, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.60511,  -0.00000,  1.30264,  1.25478,  0.00000,  1.55722, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.60511,  -0.00000,  -1.30264,  1.25478,  0.00000,  1.55722, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#600 nm for point property
        self[("TIP3P_PDB", "B3LYP", "ANO631", False, "0.0759389")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : -0.000010,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434302, ],
( 'X', 'quadrupole') : [ -3.8569700,  0.0000000,  0.0000000,  -6.0201600,  0.0000000,  -4.8724100, ],
( 'X', 'alpha' ) : [ 7.2827400,  0.0000000,  0.0000000,  6.8915600,  0.0000000,  6.9334300, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

# 500 nm
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.0911267")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.07733,  -0.00000,  0.00000,  4.31039,  0.00000,  3.79308, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.62561,  -0.00000,  1.33266,  1.36108,  0.00000,  1.61036, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.62561,  -0.00000,  -1.33266,  1.36108,  0.00000,  1.61036, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#500 nm Point property
        self[("TIP3P_PDB", "B3LYP", "ANO631", False, "0.0911267")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : -0.000010,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434302, ],
( 'X', 'quadrupole') : [ -3.8569700,  0.0000000,  0.0000000,  -6.0201600,  0.0000000,  -4.8724100, ],
( 'X', 'alpha' ) : [ 7.3285500,  0.0000000,  0.0000000,  7.0325500,  0.0000000,  7.0138000, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}
# 400 nm
        self[("TIP3P_PDB", "B3LYP", "ANO631", True, "0.1139084")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'OW' , 'charge') : -0.66229,
( 'OW' , 'dipole') : [ 0.00000,  -0.00000,  0.34276, ],
( 'OW' , 'quadrupole') : [ -3.46419,  0.00000,  -0.00000,  -5.12506,  -0.00000,  -4.31639, ],
( 'OW' , 'alpha' ) : [ 4.09199,  -0.00000,  0.00000,  4.19918,  0.00000,  3.76177, ],
( 'OW' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW1', 'charge') : 0.33114,
( 'HW1', 'dipole') : [ -0.16617,  -0.00000,  -0.11629, ],
( 'HW1', 'quadrupole') : [ -0.19639,  0.00000,  0.21589,  -0.44755,  -0.00000,  -0.27801, ],
( 'HW1', 'alpha' ) : [ 1.66159,  -0.00000,  1.38510,  1.56126,  0.00000,  1.70435, ],
( 'HW1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HW2', 'charge') : 0.33114,
( 'HW2', 'dipole') : [ 0.16617,  -0.00000,  -0.11629, ],
( 'HW2', 'quadrupole') : [ -0.19639,  -0.00000,  -0.21589,  -0.44755,  0.00000,  -0.27801, ],
( 'HW2', 'alpha' ) : [ 1.66159,  -0.00000,  -1.38510,  1.56126,  0.00000,  1.70435, ],
( 'HW2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#400 nm point property
        self[("TIP3P_PDB", "B3LYP", "ANO631", False, "0.1139084")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'X', 'charge') : -0.000010,
( 'X', 'dipole') : [ 0.0000000,  0.0000000,  0.8434302, ],
( 'X', 'quadrupole') : [ -3.8569700,  0.0000000,  0.0000000,  -6.0201600,  0.0000000,  -4.8724100, ],
( 'X', 'alpha' ) : [ 7.4151700,  0.0000000,  0.0000000,  7.3217000,  0.0000000,  7.1704700, ],
( 'X', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}

###########
#       TIP3P WORST represents the worst possible 
#       state a water can be in due to in MD using shake algortihm
#       corresponds to angle and bond elongation by 2-5%
##########
        self[("TIP3P_WORST", "HF", "ANOPVDZ", False, "0.0")] = \
                {
("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ -0.00187, -0.00000, 0.79514 ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 7.47961, -0.00000, 0.18739, 5.74008, 0.00000, 6.90488 ],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],

("O1", "quadrupole"): [ -3.77139, 0.00000, 0.03065, -5.85659, 0.00000, -4.67822 ],
("H2", "quadrupole"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "quadrupole"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],

("O1", "beta"):[ -0.44656, -0.00000, -14.33690, 0.44771, 0.00000, -1.20860, -0.00000, -2.29288, -0.00000, -5.99809 ],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
                        }

        self[("TIP3P_WORST", "HF", "ANOPVDZ", True, "0.0")] = \
                {
("O1","charge"): -0.67678,
("H2","charge"):  0.33677,
("H3","charge"):  0.34001,

("O1", "dipole"): [ -0.00749, 0.00000, 0.29811 ],
("H2", "dipole"): [ -0.15043, 0.00000, -0.13415 ],
("H3", "dipole"): [ 0.16064, -0.00000, -0.12866 ],

("O1", "quadrupole"): [ -3.57392, 0.00000, 0.01287, -4.94713, 0.00000, -4.23126 ],
("H2", "quadrupole"): [ -0.10746, 0.00000, 0.30128, -0.45904, 0.00000, -0.21041 ],
("H3", "quadrupole"): [ -0.09002, -0.00000, -0.28351, -0.45042, 0.00000, -0.23654 ],

("O1", "alpha"): [ 3.86191, -0.00000, 0.08836, 3.34296, -0.00000, 3.79683 ],
("H2", "alpha"): [ 1.82259, -0.00000, 1.12411, 1.19466, -0.00000, 1.64414 ],
("H3", "alpha"): [ 1.79512, -0.00000, -1.02508, 1.20247, 0.00000, 1.46391 ],

("O1", "beta"):[ -0.07769, -0.00000, -4.65289, 0.16970, 0.00000, -0.51643, -0.00000, 0.73722, -0.00000, 2.67808 ],
("H2", "beta"):[ -7.06240, 0.00000, -5.08211, -4.36480, 0.00000, -5.19464, 0.00000, -1.62429, 0.00000, -5.04076 ],
("H3", "beta"):[ 6.69352, -0.00000, -4.60190, 4.64281, 0.00000, 4.50247, -0.00000, -1.40581, -0.00000, -3.63541 ],
}

###########
#       Frequency dependant properties for TIP3P, 3s 2p 1d / 4s 3p 2d 1f
##########

        self[("TIP3P", "HF", "ANOPVTZ", True, "0.0238927")] = \
{
("O1","charge"): -0.689,
("H2","charge"): 0.345, 
("H3","charge"): 0.345, 

("O1", "dipole"):   [-0.000,0.000,0.269], 
("H2", "dipole"):  [-0.149,-0.000,-0.128],
("H3", "dipole"):  [0.149,0.000,-0.128],  

("O1", "alpha"):[4.973,0.000,  -0.000, 5.350,  -0.000,5.163],
("H2", "alpha"):[2.086,-0.000, 1.129,  1.233,  -0.000,1.646],
("H3", "alpha"):[2.086,-0.000, -1.129, 1.233,  -0.000,1.646],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
}

        self[("TIP3P", "HF", "ANOPVTZ", False, "0.0238927")] = \
                {
("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ -0.00000,  0.00000,  0.77468 ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 9.14435,  0.00000,  -0.00000, 7.81558, -0.00000, 8.45521],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
                        }


        self[("TIP3P", "HF", "ANOPVTZ", True, "0.0428227")] = \
{
("O1","charge"):-0.689,
("H2","charge"):0.345, 
("H3","charge"):0.345, 

("O1", "dipole"):  [-0.000,0.000,0.269], 
("H2", "dipole"): [-0.149,-0.000,-0.128],
("H3", "dipole"): [0.149,0.000,-0.128],  

("O1", "alpha"): [4.960,  0.000, -0.000, 5.301, -0.000, 5.131],
("H2", "alpha"): [2.105, -0.000,  1.146, 1.274, -0.000, 1.676],
("H3", "alpha"): [2.105, -0.000, -1.146, 1.274, -0.000, 1.676],
("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],

        }

        self[("TIP3P", "HF", "ANOPVTZ", False, "0.0428227")] = \
                {

("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ -0.00000,  0.00000,  0.77468 ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 9.16940,  0.00000,  -0.00000,7.84863,  -0.00000,  8.48327, ],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],

("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
                }
        self[("TIP3P", "HF", "ANOPVTZ", True, "0.0773571")] = \
                {
("O1","charge"): -0.689,
("H2","charge"): 0.345, 
("H3","charge"): 0.345, 

("O1", "dipole"):   [-0.000,0.000,0.269], 
("H2", "dipole"):  [-0.149,-0.000,-0.128],
("H3", "dipole"):  [0.149,0.000,-0.128],  

("O1", "alpha"):[4.951,0.000,  -0.000,5.229, -0.000,5.090],
("H2", "alpha"):[2.151,-0.000,1.187, 1.366, -0.000,1.744],
("H3", "alpha"):[2.151,-0.000,  -1.187,1.366, -0.000,1.744],

("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
        }

        self[("TIP3P", "HF", "ANOPVTZ", False, "0.0773571")] = \
                {

("O1","charge"): 0.,
("H2","charge"): 0.,
("H3","charge"): 0.,

("O1", "dipole"): [ -0.00000,  0.00000,  0.77468 ],
("H2", "dipole"): [ 0.0,   0.0,   0.0,    ],
("H3", "dipole"): [ 0.0,   0.0,   0.0,    ],

("O1", "alpha"): [ 9.25300,  0.00000,  -0.00000,7.96160,  -0.00000, 8.57770 ],
("H2", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],
("H3", "alpha"): [ 0.00,   0.0, 0.0,  0.0,  0.0,  0.0,  ],

("O1", "beta"):[0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
("H2", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
("H3", "beta"):[0., 0., 0., 0., 0. ,0., 0., 0., 0., 0.],
}



###########
#       Ano-1 basis set 3s2p1d contr on H / 4s3p2d1f contr on O
#       CAMB3LYP functional
##########



########################
# Paranitro at 6-31+G* Basis, 2s H, 4s3p1d O
        self[("PNA", "HF", "ANO631", True, "0.0")] = \
                {
( 'N1', "charge") :  0.65444 ,
( 'N2', "charge") :  -0.50087 ,
( 'C3', "charge") :  -0.03722 ,
( 'C4', "charge") :  -0.04367 ,
( 'C5', "charge") :  -0.20541 ,
( 'C6', "charge") :  0.23493 ,
( 'C7', "charge") :  -0.20541 ,
( 'C8', "charge") :  -0.04367 ,
( 'H9', "charge") :  0.16161 ,
( 'H10', "charge") :  0.12276 ,
( 'H11', "charge") :  0.25699 ,
( 'H12', "charge") :  0.25699 ,
( 'H13', "charge") :  0.12276 ,
( 'H14', "charge") :  0.16161 ,
( 'O15', "charge") :  -0.46791 ,
( 'O16', "charge") :  -0.46792 ,
( 'N1', "dipole") : [ -0.00063, 0.00654, 0.22999 ],
( 'N2', "dipole") : [ -0.00005, 0.00051, 0.19828 ],
( 'C3', "dipole") : [ 0.00023, -0.00243, -0.10677 ],
( 'C4', "dipole") : [ 0.18324, 0.01449, -0.13405 ],
( 'C5', "dipole") : [ 0.16741, 0.01940, 0.13095 ],
( 'C6', "dipole") : [ 0.00024, -0.00253, -0.12835 ],
( 'C7', "dipole") : [ -0.16804, -0.01276, 0.13095 ],
( 'C8', "dipole") : [ -0.18266, -0.02057, -0.13404 ],
( 'H9', "dipole") : [ 0.21152, 0.01744, -0.12081 ],
( 'H10', "dipole") : [ 0.22455, 0.02479, 0.13775 ],
( 'H11', "dipole") : [ 0.17520, 0.01970, 0.08889 ],
( 'H12', "dipole") : [ -0.17575, -0.01393, 0.08889 ],
( 'H13', "dipole") : [ -0.22517, -0.01831, 0.13776 ],
( 'H14', "dipole") : [ -0.21099, -0.02306, -0.12081 ],
( 'O15', "dipole") : [ 0.40570, 0.03476, -0.16323 ],
( 'O16', "dipole") : [ -0.40491, -0.04293, -0.16322 ],
( 'N1', "quadrupole") : [ -1.36535, 0.16542, -0.00285, -3.07553, 0.02962, -1.87906 ],
( 'N2', "quadrupole") : [ -3.22208, 0.17836, -0.00520, -5.06601, 0.05429, -2.99305 ],
( 'C3', "quadrupole") : [ -2.19650, 0.21681, -0.00301, -4.43817, 0.03131, -3.10849 ],
( 'C4', "quadrupole") : [ -2.86069, 0.13506, 0.25441, -4.19238, 0.06728, -2.39707 ],
( 'C5', "quadrupole") : [ -3.02675, 0.16341, -0.25721, -4.78064, 0.02699, -2.61229 ],
( 'C6', "quadrupole") : [ -1.98663, 0.16841, -0.00347, -3.72781, 0.03614, -2.17610 ],
( 'C7', "quadrupole") : [ -3.02907, 0.17542, 0.24741, -4.77834, 0.07535, -2.61231 ],
( 'C8', "quadrupole") : [ -2.85835, 0.12300, -0.26255, -4.19471, 0.01774, -2.39707 ],
( 'H9', "quadrupole") : [ -0.16949, 0.03041, -0.22170, -0.53873, -0.01809, -0.40795 ],
( 'H10', "quadrupole") : [ -0.20099, 0.04247, 0.23628, -0.58134, 0.02533, -0.47141 ],
( 'H11', "quadrupole") : [ -0.16497, 0.03784, 0.19766, -0.48820, 0.02114, -0.43950 ],
( 'H12', "quadrupole") : [ -0.16252, 0.02516, -0.19808, -0.49065, -0.01679, -0.43949 ],
( 'H13', "quadrupole") : [ -0.19887, 0.03152, -0.23679, -0.58346, -0.02001, -0.47140 ],
( 'H14', "quadrupole") : [ -0.17146, 0.04064, 0.22109, -0.53675, 0.02435, -0.40796 ],
( 'O15', "quadrupole") : [ -3.56502, 0.05068, -0.45998, -4.21549, -0.04647, -4.33418 ],
( 'O16', "quadrupole") : [ -3.56959, 0.07427, 0.46042, -4.21096, 0.04174, -4.33423 ],
( 'N1', "alpha") : [ 9.07865, 0.60521, -0.02534, 2.82290, 0.26444, 13.88995 ],
( 'N2', "alpha") : [ 2.94020, -0.20201, -0.01658, 5.03007, 0.17401, 13.13939 ],
( 'C3', "alpha") : [ 9.52554, 0.43015, -0.02918, 5.07856, 0.30530, 18.05289 ],
( 'C4', "alpha") : [ 7.95169, 0.31972, 2.83947, 5.36540, 0.45909, 13.22288 ],
( 'C5', "alpha") : [ 8.66851, 0.18595, -1.96447, 6.26625, -0.00580, 13.90440 ],
( 'C6', "alpha") : [ 9.14986, 0.42849, -0.02062, 4.71996, 0.21565, 14.10092 ],
( 'C7', "alpha") : [ 8.65137, 0.27561, 1.92991, 6.28365, 0.36726, 13.90433 ],
( 'C8', "alpha") : [ 7.97750, 0.18570, -2.87492, 5.33948, -0.08846, 13.22289 ],
( 'H9', "alpha") : [ 2.80113, 0.20580, -1.29228, 0.35421, -0.10606, 1.09222 ],
( 'H10', "alpha") : [ 3.24712, 0.31436, 1.70224, 0.41806, 0.18586, 1.34910 ],
( 'H11', "alpha") : [ 1.68368, 0.16504, 1.42337, 0.42716, 0.17578, 1.66285 ],
( 'H12', "alpha") : [ 1.69982, 0.08115, -1.43081, 0.41093, -0.09776, 1.66285 ],
( 'H13', "alpha") : [ 3.26227, 0.23584, -1.70657, 0.40287, -0.14082, 1.34914 ],
( 'H14', "alpha") : [ 2.78966, 0.26534, 1.28887, 0.36572, 0.14131, 1.09217 ],
( 'O15', "alpha") : [ 7.40285, 0.32410, -3.98665, 3.01495, -0.29732, 6.43266 ],
( 'O16', "alpha") : [ 7.36557, 0.51767, 3.97056, 3.05243, 0.46526, 6.43256 ],
( 'N1', "beta") : [ -0.25895, 0.94924, 39.31603, 0.16318, 3.57912, 0.54225, 0.00325, 1.47546, -5.67702, -246.75427 ],
( 'N2', "beta") : [ -0.03593, 0.13691, 8.06920, -0.13318, -1.56742, 0.31843, 1.62958, 19.84769, -3.32621, -88.23466 ],
( 'C3', "beta") : [ -0.20865, 0.86367, 37.69830, 0.19171, 3.97448, 0.59126, -0.69513, -6.26065, -6.19543, -290.30300 ],
( 'C4', "beta") : [ 43.48504, 0.63820, 4.12776, 14.67592, 0.15430, -21.95402, 4.72031, -2.88842, -3.97596, -90.06772 ],
( 'C5', "beta") : [ 19.23357, -1.83922, 24.90386, 16.69010, 1.74064, 21.83062, 6.48257, 6.86321, -0.33737, -85.47661 ],
( 'C6', "beta") : [ -0.13558, 0.41026, 17.46020, -0.00774, 0.48647, 0.36872, 1.08642, 11.64590, -3.85107, -130.72794 ],
( 'C7', "beta") : [ -19.60135, 3.02055, 24.84453, -16.54974, 1.86944, -21.37097, -5.29579, 6.92243, -4.47775, -85.47814 ],
( 'C8', "beta") : [ -43.53449, -0.45740, 3.92874, -14.58682, 1.41507, 22.30682, -5.31970, -2.68938, 0.26476, -90.06945 ],
( 'H9', "beta") : [ 11.17904, 0.74483, -4.76709, -0.45845, -0.52409, -1.78227, 0.19488, 0.86191, -0.05100, 5.12479 ],
( 'H10', "beta") : [ 13.68436, 1.18187, 8.17264, -0.27459, 0.97683, 2.08120, 0.28411, -0.84228, 0.27369, 3.42380 ],
( 'H11', "beta") : [ 5.17947, 0.60009, 4.52317, -1.10712, 0.17215, -2.50558, 0.21074, 2.74526, -0.49628, -0.88160 ],
( 'H12', "beta") : [ -5.26556, -0.33669, 4.55320, 1.11366, 0.03768, 2.55428, 0.36056, 2.71489, -0.01150, -0.88179 ],
( 'H13', "beta") : [ -13.80813, -0.80643, 8.20073, 0.36471, 0.84207, -2.09527, -0.30314, -0.87060, -0.12651, 3.42381 ],
( 'H14', "beta") : [ -11.11279, -0.95966, -4.76879, 0.40868, -0.49785, 1.75968, -0.15268, 0.86379, 0.28835, 5.12465 ],
( 'O15', "beta") : [ -7.92504, -1.75347, 10.12756, 3.32585, 1.92372, 34.87658, 1.22463, -3.11409, 1.87992, -71.19162 ],
( 'O16', "beta") : [ 7.76601, 2.24792, 10.21844, -3.15423, 1.13173, -34.60027, -1.89895, -3.20700, -4.77784, -71.19013 ],
}

        self[("PNA", "HF", "ANO631", False, "0.0")] = \
                {
( 'C3', "charge") :  -0.00000 ,
( 'C3', "dipole") : [ 0.00774, -0.08127, -3.15541 ],
( 'C3', "quadrupole") : [ -28.74832, 1.65888, -0.03357, -45.89917, 0.34992, -31.48155 ],
( 'C3', "alpha") : [ 94.19542, 4.33812, -0.19303, 49.35263, 2.01773, 134.51120 ],
( 'C3', "beta") : [ -1.35899, 4.64066, 196.60848, 0.66196, 15.71434, 2.92148, 2.53165, 34.06812, -30.59722, -1234.15986 ],
}

########################
# Paranitro at ANOMAX, 5s4p3d H, 5s4p3d2f O, 7 6 4 3 S
        self[("PNA", "HF", "ANOMAX", True, "0.0")] = \
                {
( 'N1', "charge") :  0.67684 ,
( 'N2', "charge") :  -0.51506 ,
( 'C3', "charge") :  -0.03783 ,
( 'C4', "charge") :  -0.04816 ,
( 'C5', "charge") :  -0.21314 ,
( 'C6', "charge") :  0.24046 ,
( 'C7', "charge") :  -0.21314 ,
( 'C8', "charge") :  -0.04816 ,
( 'H9', "charge") :  0.16625 ,
( 'H10', "charge") :  0.12629 ,
( 'H11', "charge") :  0.26245 ,
( 'H12', "charge") :  0.26245 ,
( 'H13', "charge") :  0.12629 ,
( 'H14', "charge") :  0.16625 ,
( 'O15', "charge") :  -0.47588 ,
( 'O16', "charge") :  -0.47589 ,
( 'N1', "dipole") : [ -0.00065, 0.00676, 0.23582 ],
( 'N2', "dipole") : [ -0.00007, 0.00074, 0.21691 ],
( 'C3', "dipole") : [ 0.00022, -0.00231, -0.10101 ],
( 'C4', "dipole") : [ 0.19479, 0.01509, -0.15531 ],
( 'C5', "dipole") : [ 0.18965, 0.02190, 0.14695 ],
( 'C6', "dipole") : [ 0.00021, -0.00224, -0.12330 ],
( 'C7', "dipole") : [ -0.19035, -0.01452, 0.14695 ],
( 'C8', "dipole") : [ -0.19412, -0.02218, -0.15530 ],
( 'H9', "dipole") : [ 0.19879, 0.01647, -0.11078 ],
( 'H10', "dipole") : [ 0.20576, 0.02293, 0.13436 ],
( 'H11', "dipole") : [ 0.17532, 0.01962, 0.07413 ],
( 'H12', "dipole") : [ -0.17585, -0.01404, 0.07413 ],
( 'H13', "dipole") : [ -0.20636, -0.01656, 0.13436 ],
( 'H14', "dipole") : [ -0.19830, -0.02159, -0.11078 ],
( 'O15', "dipole") : [ 0.39123, 0.03407, -0.13432 ],
( 'O16', "dipole") : [ -0.39058, -0.04086, -0.13431 ],
( 'N1', "quadrupole") : [ -1.27997, 0.16771, -0.00277, -3.01390, 0.02879, -1.85783 ],
( 'N2', "quadrupole") : [ -3.29814, 0.14993, -0.00440, -4.84815, 0.04590, -3.11084 ],
( 'C3', "quadrupole") : [ -2.11391, 0.22564, -0.00308, -4.44679, 0.03199, -3.08892 ],
( 'C4', "quadrupole") : [ -2.97917, 0.12295, 0.29055, -4.17645, 0.06911, -2.44703 ],
( 'C5', "quadrupole") : [ -3.15402, 0.14466, -0.31611, -4.72799, 0.01864, -2.66691 ],
( 'C6', "quadrupole") : [ -1.89120, 0.18245, -0.00360, -3.77752, 0.03750, -2.15419 ],
( 'C7', "quadrupole") : [ -3.15684, 0.15927, 0.30683, -4.72519, 0.07833, -2.66694 ],
( 'C8', "quadrupole") : [ -2.97650, 0.10917, -0.29838, -4.17911, 0.01267, -2.44702 ],
( 'H9', "quadrupole") : [ -0.19364, 0.02553, -0.21956, -0.51170, -0.01873, -0.41678 ],
( 'H10', "quadrupole") : [ -0.23458, 0.03655, 0.23922, -0.55319, 0.02385, -0.51653 ],
( 'H11', "quadrupole") : [ -0.11628, 0.04318, 0.20054, -0.48984, 0.01997, -0.50280 ],
( 'H12', "quadrupole") : [ -0.11366, 0.02959, -0.20068, -0.49247, -0.01848, -0.50279 ],
( 'H13', "quadrupole") : [ -0.23244, 0.02549, -0.23939, -0.55533, -0.02202, -0.51653 ],
( 'H14', "quadrupole") : [ -0.19559, 0.03563, 0.21912, -0.50974, 0.02331, -0.41679 ],
( 'O15', "quadrupole") : [ -3.58208, 0.04857, -0.39524, -4.19364, -0.04165, -4.36768 ],
( 'O16', "quadrupole") : [ -3.58603, 0.06898, 0.39594, -4.18973, 0.03417, -4.36773 ],
( 'N1', "alpha") : [ 8.85708, 0.56399, -0.02558, 3.02760, 0.26696, 14.21226 ],
( 'N2', "alpha") : [ 3.63344, -0.11389, -0.01871, 4.81283, 0.19625, 13.72524 ],
( 'C3', "alpha") : [ 9.57203, 0.42511, -0.02906, 5.17849, 0.30398, 18.07423 ],
( 'C4', "alpha") : [ 8.33388, 0.34646, 2.71004, 5.43874, 0.44926, 13.38984 ],
( 'C5', "alpha") : [ 9.09294, 0.24308, -1.89180, 6.11500, 0.00867, 14.06172 ],
( 'C6', "alpha") : [ 9.19644, 0.41109, -0.02121, 4.94782, 0.22179, 14.51301 ],
( 'C7', "alpha") : [ 9.07629, 0.33014, 1.85580, 6.13189, 0.36770, 14.06166 ],
( 'C8', "alpha") : [ 8.35850, 0.21870, -2.74599, 5.41403, -0.07355, 13.38984 ],
( 'H9', "alpha") : [ 3.31851, 0.19507, -1.30513, 0.98045, -0.11490, 1.39996 ],
( 'H10', "alpha") : [ 3.75088, 0.29556, 1.78908, 1.13910, 0.18623, 1.73429 ],
( 'H11', "alpha") : [ 2.04382, 0.13429, 1.57965, 1.13654, 0.19154, 2.42540 ],
( 'H12', "alpha") : [ 2.06108, 0.04460, -1.58723, 1.11918, -0.11197, 2.42541 ],
( 'H13', "alpha") : [ 3.76685, 0.21280, -1.79192, 1.12309, -0.15696, 1.73434 ],
( 'H14', "alpha") : [ 3.30696, 0.25503, 1.30316, 0.99205, 0.13507, 1.39991 ],
( 'O15', "alpha") : [ 7.58493, 0.27858, -3.85659, 3.70363, -0.27971, 7.33907 ],
( 'O16', "alpha") : [ 7.54894, 0.46549, 3.83953, 3.73983, 0.45782, 7.33899 ],
( 'N1', "beta") : [ -0.24674, 0.92455, 38.29827, 0.16646, 3.59707, 0.51686, -0.13007, 0.41164, -5.41058, -240.02499 ],
( 'N2', "beta") : [ -0.09323, 0.30971, 13.63737, -0.10240, -1.10136, 0.27728, 1.73661, 21.10924, -2.89637, -68.03765 ],
( 'C3', "beta") : [ -0.19119, 0.82995, 36.06629, 0.17032, 3.61746, 0.57823, -0.61882, -4.91164, -6.05858, -282.41103 ],
( 'C4', "beta") : [ 49.50602, 0.75005, 4.76424, 16.55497, 0.11890, -21.24975, 5.39834, -2.27141, -3.91262, -90.36281 ],
( 'C5', "beta") : [ 25.32056, -1.82291, 23.69053, 18.89561, 1.57351, 20.97894, 7.25296, 7.07445, -0.32262, -80.59897 ],
( 'C6', "beta") : [ -0.14140, 0.41780, 17.80820, 0.00299, 0.59925, 0.34927, 1.02773, 11.13103, -3.64813, -123.53180 ],
( 'C7', "beta") : [ -25.67794, 2.94304, 23.60812, -18.76454, 1.83468, -20.53731, -6.01914, 7.15695, -4.30242, -80.60047 ],
( 'C8', "beta") : [ -49.55801, -0.53383, 4.55380, -16.46948, 1.43758, 21.60355, -5.96853, -2.06072, 0.19254, -90.36460 ],
( 'H9', "beta") : [ 18.21993, 0.71348, -6.19506, 1.67978, -0.57382, -0.52846, 0.97039, -0.20267, 0.16034, 5.60356 ],
( 'H10', "beta") : [ 21.53655, 1.14499, 9.35277, 2.31142, 0.81994, 3.39643, 1.54534, 1.20875, 0.37103, 8.47261 ],
( 'H11', "beta") : [ 8.09066, 0.30432, 6.67720, 1.32428, 0.11738, 1.42278, 1.36373, 5.33020, -0.01077, 11.87251 ],
( 'H12', "beta") : [ -8.21543, 0.06014, 6.71207, -1.34440, -0.04633, -1.39465, -0.20969, 5.29479, -0.28097, 11.87246 ],
( 'H13', "beta") : [ -21.68092, -0.71181, 9.36491, -2.24348, 0.75906, -3.40509, -1.17742, 1.19650, -0.28074, 8.47275 ],
( 'H14', "beta") : [ -18.13305, -0.99439, -6.21557, -1.72446, -0.46764, 0.48853, -1.13045, -0.18188, 0.25776, 5.60350 ],
( 'O15', "beta") : [ -1.93253, -1.89134, 6.13173, 5.50979, 1.78288, 37.28200, 1.82731, -5.48104, 2.07970, -79.42248 ],
( 'O16', "beta") : [ 1.83767, 2.20349, 6.23528, -5.34219, 0.94427, -36.99991, -2.95012, -5.58695, -5.03888, -79.42096 ],
}
        self[("PNA", "HF", "ANOMAX", False, "0.0")] = \
                {
( 'C3', "charge") :  -0.00000 ,
( 'C3', "dipole") : [ 0.00750, -0.07874, -3.06787 ],
( 'C3', "quadrupole") : [ -29.10404, 1.57529, -0.03103, -45.39075, 0.32335, -32.04731 ],
( 'C3', "alpha") : [ 99.50257, 4.30612, -0.19596, 55.00027, 2.04819, 141.22515 ],
( 'C3', "beta") : [ -1.35905, 4.64724, 194.49015, 0.62467, 15.01283, 2.77869, 2.91818, 39.21726, -29.10131, -1162.87835 ],
}


########################
# Sulfur at ANOMAX, 5s4p3d H, 5s4p3d2f O, 7 6 4 3 S
        self[("SULFACID", "HF", "ANOMAX", True, "0.0")] = \
                {
( 'O1', "charge") :  -0.69023 ,
( 'O2', "charge") :  -0.88433 ,
( 'O3', "charge") :  -0.84863 ,
( 'O4', "charge") :  -0.70849 ,
( 'S5', "charge") :  2.38084 ,
( 'H6', "charge") :  0.37518 ,
( 'H7', "charge") :  0.37565 ,
( 'O1', "dipole") : [ -0.21659, -0.02003, -0.09954 ],
( 'O2', "dipole") : [ 0.03493, -0.01601, 0.02440 ],
( 'O3', "dipole") : [ -0.00068, -0.01229, -0.00415 ],
( 'O4', "dipole") : [ -0.12403, 0.17831, 0.11774 ],
( 'S5', "dipole") : [ 0.07157, -0.04802, -0.02985 ],
( 'H6', "dipole") : [ 0.14074, -0.01769, 0.04182 ],
( 'H7', "dipole") : [ 0.08568, -0.08261, -0.08710 ],
( 'O1', "quadrupole") : [ -3.73625, 0.06547, 0.28038, -4.28819, 0.11445, -4.15058 ],
( 'O2', "quadrupole") : [ -4.20757, -0.01204, -0.10611, -4.58521, 0.00089, -4.48863 ],
( 'O3', "quadrupole") : [ -4.17184, 0.00809, 0.07452, -4.46481, 0.00024, -4.38202 ],
( 'O4', "quadrupole") : [ -4.12701, -0.33915, -0.16756, -3.97495, 0.11615, -4.23677 ],
( 'S5', "quadrupole") : [ -2.89781, -0.06922, -0.09609, -3.86437, -0.02293, -3.42908 ],
( 'H6', "quadrupole") : [ 0.02081, 0.02934, 0.18254, -0.45672, -0.03898, -0.33597 ],
( 'H7', "quadrupole") : [ -0.22255, -0.17939, -0.12785, -0.22466, 0.18579, -0.31899 ],
( 'O1', "alpha") : [ 3.51425, -0.08975, -0.03313, 6.41831, 2.38832, 5.36512 ],
( 'O2', "alpha") : [ 7.48007, 0.13163, -2.30002, 3.93320, -0.06999, 5.36829 ],
( 'O3', "alpha") : [ 7.66296, 0.00829, 2.33460, 3.86751, -0.01588, 5.21921 ],
( 'O4', "alpha") : [ 3.49875, -0.21143, 0.08489, 6.69419, -2.39766, 5.43204 ],
( 'S5', "alpha") : [ 8.91885, -0.08143, -0.03457, 9.18131, -0.19378, 9.17023 ],
( 'H6', "alpha") : [ 2.00390, 0.53536, 0.90440, 1.12549, 0.43775, 1.54153 ],
( 'H7', "alpha") : [ 1.32598, -0.86242, -0.14837, 2.51405, 0.26649, 0.95392 ],
( 'O1', "beta") : [ -1.19788, 4.14521, 4.57258, -0.28367, 0.75174, 0.28927, 9.19208, -2.29214, 1.35563, 8.00317 ],
( 'O2', "beta") : [ 14.01741, 0.12744, 0.62830, 3.49678, -0.17315, 2.15251, 1.51731, -3.98532, 0.12590, -12.74794 ],
( 'O3', "beta") : [ -13.37306, 0.13957, 1.84493, -3.81899, 0.19343, -1.54033, 0.45102, -3.05382, 0.23543, -10.95478 ],
( 'O4', "beta") : [ -0.93973, -4.42760, 3.25769, 0.63190, 0.68107, 0.00811, -11.20030, -1.78666, -1.16994, 9.63899 ],
( 'S5', "beta") : [ -1.04301, -0.16879, -3.11356, 0.99276, 0.33016, 0.50892, -2.18914, 2.94148, 1.16527, 0.83186 ],
( 'H6', "beta") : [ 7.29777, 3.14960, 4.05254, 1.55985, 1.45162, 3.29223, 5.92988, 1.55736, 2.45244, 7.80573 ],
( 'H7', "beta") : [ 4.07728, -3.64612, 0.77142, 3.55332, 0.28192, 0.41583, -14.71201, -1.34022, -2.23497, 1.77583 ],
}
        self[("SULFACID", "HF", "ANOMAX", False, "0.0")] = \
                {
( 'S5', "charge") :  0.000 ,
( 'S5', "dipole") : [ -0.926, 0.361, -1.046 ],
( 'S5', "quadrupole") : [ -19.342, -0.497, 0.040, -21.859, 0.356, -21.342 ],
( 'S5', "alpha") : [ 34.405, -0.570, 0.808, 33.734, 0.415, 33.050 ],
( 'S5', "beta") : [ 8.839, -0.681, 12.014, 6.132, 3.517, 5.127, -11.011, -7.959, 1.930, 4.353 ],
}


########################
# Sulfur at ANO631, 2s H, 4s 3p 1d O, 5s 4p 1d S
        self[("SULFACID", "HF", "ANO631", True, "0.0")] = \
                {
( 'O1', "charge") :  -0.66248 ,
( 'O2', "charge") :  -0.85161 ,
( 'O3', "charge") :  -0.81368 ,
( 'O4', "charge") :  -0.68032 ,
( 'S5', "charge") :  2.26987 ,
( 'H6', "charge") :  0.36955 ,
( 'H7', "charge") :  0.36867 ,
( 'O1', "dipole") : [ -0.24138, 0.00133, -0.08855 ],
( 'O2', "dipole") : [ 0.08946, -0.01199, -0.00881 ],
( 'O3', "dipole") : [ -0.06218, -0.00973, -0.03164 ],
( 'O4', "dipole") : [ -0.13736, 0.16894, 0.15038 ],
( 'S5', "dipole") : [ 0.06302, -0.04415, -0.02069 ],
( 'H6', "dipole") : [ 0.14274, 0.01523, 0.06789 ],
( 'H7', "dipole") : [ 0.08944, -0.11427, -0.06563 ],
( 'O1', "quadrupole") : [ -3.69566, 0.10354, 0.33752, -4.23968, 0.19690, -4.09767 ],
( 'O2', "quadrupole") : [ -4.11086, -0.00582, -0.16008, -4.59383, 0.00560, -4.47534 ],
( 'O3', "quadrupole") : [ -4.05032, 0.01309, 0.15495, -4.48657, -0.00101, -4.35710 ],
( 'O4', "quadrupole") : [ -4.14375, -0.40546, -0.17715, -3.83254, 0.08408, -4.22706 ],
( 'S5', "quadrupole") : [ -3.06519, -0.07326, -0.07444, -4.08027, -0.01586, -3.64654 ],
( 'H6', "quadrupole") : [ -0.09381, 0.03552, 0.16076, -0.45203, -0.00808, -0.37049 ],
( 'H7', "quadrupole") : [ -0.29721, -0.16068, -0.09674, -0.25655, 0.14404, -0.37854 ],
( 'O1', "alpha") : [ 3.01121, -0.15613, -0.10486, 6.10581, 2.54062, 4.97255 ],
( 'O2', "alpha") : [ 7.35994, 0.14241, -2.68849, 3.02821, -0.05498, 4.73665 ],
( 'O3', "alpha") : [ 7.46454, 0.02056, 2.68239, 2.97853, -0.01345, 4.54987 ],
( 'O4', "alpha") : [ 3.04322, -0.18230, 0.12428, 6.30126, -2.62799, 5.12251 ],
( 'S5', "alpha") : [ 8.79002, -0.05728, -0.03835, 9.08649, -0.21403, 9.04315 ],
( 'H6', "alpha") : [ 1.67523, 0.49384, 0.93175, 0.59810, 0.43562, 1.04128 ],
( 'H7', "alpha") : [ 0.90239, -0.91373, -0.22128, 1.97477, 0.29217, 0.51466 ],
( 'O1', "beta") : [ 0.09245, 3.14837, 4.28801, -0.44767, 0.25118, -0.14784, 7.74042, -3.41002, 0.32361, 6.85361 ],
( 'O2', "beta") : [ 10.26366, -0.04652, 0.81976, 2.52173, -0.13969, 1.62570, 1.08822, -3.51090, 0.10288, -9.44733 ],
( 'O3', "beta") : [ -9.62912, 0.06154, 1.35496, -2.41543, 0.20117, -1.20492, 0.25698, -2.83063, 0.19451, -8.07041 ],
( 'O4', "beta") : [ -0.41939, -3.88791, 3.26245, 0.06312, 0.95305, 0.37233, -8.37294, -2.45584, -0.65706, 8.16404 ],
( 'S5', "beta") : [ -0.46375, -0.50858, -3.94821, 1.23758, -0.02998, 0.61198, -2.00382, 3.01995, 1.01565, 0.37314 ],
( 'H6', "beta") : [ 5.11242, 2.09937, 3.74173, -0.07852, 1.31735, 2.13930, 2.59766, 0.38000, 1.53612, 3.64744 ],
( 'H7', "beta") : [ 1.48932, -2.29368, 0.22308, 3.16506, 1.03319, 0.01039, -8.67823, -1.54181, -0.94273, 1.81919 ],
}

        self[("SULFACID", "HF", "ANO631", False, "0.0")] = \
                {
( 'S5', "charge") :  0.00000 ,
( 'S5', "dipole") : [ -0.95102, 0.37432, -1.00532 ],
( 'S5', "quadrupole") : [ -19.45679, -0.49308, 0.14482, -21.94147, 0.40567, -21.55274 ],
( 'S5', "alpha") : [ 32.24655, -0.65263, 0.68544, 30.07317, 0.35795, 29.98067 ],
( 'S5', "beta") : [ 6.44559, -1.42742, 9.74178, 4.04586, 3.58628, 3.40695, -7.37171, -10.34923, 1.57297, 3.33969 ],
}

#ANO631 is 4s 3p 1s on C/N/O and 2s on H
#PMMA_N is first monomer with lacks last hydrogen
        self[("PMMA_N", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : 0.11437,
( 'CA' , 'dipole') : [ -0.05953,  0.02462,  -0.17322, ],
( 'CA' , 'quadrupole') : [ -1.81175,  -0.10393,  -0.01421,  -1.89593,  0.03295,  -2.45931, ],
( 'CA' , 'alpha' ) : [ 1.40025,  3.33168,  0.25813,  8.55867,  4.78894,  4.67763, ],
( 'CA' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB1', 'charge') : -0.40517,
( 'CB1', 'dipole') : [ -0.19475,  0.09780,  0.07520, ],
( 'CB1', 'quadrupole') : [ -3.59312,  -0.11896,  -0.12785,  -3.76255,  0.04833,  -3.72534, ],
( 'CB1', 'alpha' ) : [ 2.37257,  -0.18822,  -0.57278,  11.09377,  7.82287,  3.60705, ],
( 'CB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB2', 'charge') : 0.47145,
( 'CB2', 'dipole') : [ -0.05280,  -0.02779,  0.10719, ],
( 'CB2', 'quadrupole') : [ -1.78095,  -0.00383,  -0.07756,  -3.62978,  -0.04581,  -2.54583, ],
( 'CB2', 'alpha' ) : [ -1.00104,  3.50028,  2.95375,  8.93335,  8.80164,  6.25874, ],
( 'CB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG1', 'charge') : -0.15843,
( 'OG1', 'dipole') : [ 0.02573,  -0.08093,  -0.25488, ],
( 'OG1', 'quadrupole') : [ -3.53251,  -0.02815,  -0.11355,  -3.51849,  0.21775,  -2.94617, ],
( 'OG1', 'alpha' ) : [ -1.64610,  2.23123,  2.22457,  8.98174,  7.09970,  7.50340, ],
( 'OG1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG2', 'charge') : -0.38628,
( 'OG2', 'dipole') : [ 0.21899,  0.12469,  0.13162, ],
( 'OG2', 'quadrupole') : [ -3.61079,  0.34493,  -0.01656,  -3.54551,  0.15736,  -3.59450, ],
( 'OG2', 'alpha' ) : [ 3.85672,  3.54384,  2.83390,  9.56677,  6.04351,  4.34729, ],
( 'OG2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CD' , 'charge') : -0.39846,
( 'CD' , 'dipole') : [ -0.22970,  -0.13110,  -0.09043, ],
( 'CD' , 'quadrupole') : [ -3.24210,  0.26214,  0.14496,  -3.42773,  0.13724,  -3.63840, ],
( 'CD' , 'alpha' ) : [ 2.36766,  3.76096,  1.22085,  9.63410,  4.64251,  2.98817, ],
( 'CD' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'C'  , 'charge') : -0.21526,
( 'C'  , 'dipole') : [ 0.27425,  -0.08002,  0.19604, ],
( 'C'  , 'quadrupole') : [ -3.09483,  0.35905,  0.01735,  -2.89807,  0.05954,  -3.24017, ],
( 'C'  , 'alpha' ) : [ 0.67561,  3.42692,  2.27091,  12.45121,  4.84047,  2.75562, ],
( 'C'  , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HN' , 'charge') : -0.04548,
( 'HN' , 'dipole') : [ 0.64809,  0.12959,  -0.30340, ],
( 'HN' , 'quadrupole') : [ -2.29405,  -0.31919,  -0.52773,  -2.96100,  -0.07444,  -3.51435, ],
( 'HN' , 'alpha' ) : [ -1.52880,  1.85152,  -2.35509,  7.17132,  7.05669,  2.86354, ],
( 'HN' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB1', 'charge') : -0.32672,
( 'HB1', 'dipole') : [ -0.43868,  0.10315,  0.23000, ],
( 'HB1', 'quadrupole') : [ -3.35943,  -0.00095,  0.34483,  -3.95445,  0.22488,  -3.04335, ],
( 'HB1', 'alpha' ) : [ -0.33051,  1.67708,  3.90512,  5.03851,  6.03733,  6.37822, ],
( 'HB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB2', 'charge') : 0.13468,
( 'HB2', 'dipole') : [ -0.01092,  0.29442,  -0.10333, ],
( 'HB2', 'quadrupole') : [ -0.59928,  -0.01802,  -0.01467,  0.00799,  -0.21718,  -0.50137, ],
( 'HB2', 'alpha' ) : [ -0.54316,  0.86735,  0.08147,  4.03139,  0.25064,  0.04243, ],
( 'HB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB3', 'charge') : 0.13811,
( 'HB3', 'dipole') : [ -0.21961,  -0.02083,  -0.21942, ],
( 'HB3', 'quadrupole') : [ -0.24672,  0.04920,  0.31880,  -0.56676,  0.03182,  -0.19045, ],
( 'HB3', 'alpha' ) : [ 0.58095,  0.34759,  1.63845,  1.09209,  1.39728,  0.79618, ],
( 'HB3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD1', 'charge') : 0.11935,
( 'HD1', 'dipole') : [ -0.10565,  -0.14281,  0.26659, ],
( 'HD1', 'quadrupole') : [ -0.52492,  0.11610,  -0.15884,  -0.47869,  -0.26392,  -0.12625, ],
( 'HD1', 'alpha' ) : [ -2.23656,  1.37398,  -0.68251,  2.26946,  2.20402,  1.29883, ],
( 'HD1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD2', 'charge') : 0.12798,
( 'HD2', 'dipole') : [ 0.07606,  0.30036,  0.05277, ],
( 'HD2', 'quadrupole') : [ -0.55532,  0.17412,  0.03529,  0.03367,  0.09434,  -0.54758, ],
( 'HD2', 'alpha' ) : [ 1.26661,  0.05985,  -0.07959,  3.33883,  0.10893,  0.95539, ],
( 'HD2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD3', 'charge') : 0.14179,
( 'HD3', 'dipole') : [ 0.12892,  0.21041,  -0.13988, ],
( 'HD3', 'quadrupole') : [ -0.42856,  0.18813,  -0.12005,  -0.19824,  -0.18710,  -0.47385, ],
( 'HD3', 'alpha' ) : [ -2.25715,  1.68903,  -0.17382,  3.14842,  1.31037,  0.33991, ],
( 'HD3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H1' , 'charge') : -0.07036,
( 'H1' , 'dipole') : [ -0.34434,  -0.02515,  -0.16266, ],
( 'H1' , 'quadrupole') : [ -0.99505,  -0.69968,  -0.13581,  -1.22510,  -0.00808,  -1.25753, ],
( 'H1' , 'alpha' ) : [ 1.80204,  -4.16431,  -2.82730,  1.94736,  4.06522,  1.33172, ],
( 'H1' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H2' , 'charge') : 0.48823,
( 'H2' , 'dipole') : [ -0.32754,  -0.37421,  0.09791, ],
( 'H2' , 'quadrupole') : [ -1.20031,  0.07233,  0.13009,  -4.19247,  0.18662,  -2.48105, ],
( 'H2' , 'alpha' ) : [ 2.44667,  3.56913,  4.76453,  5.17492,  9.62153,  7.01924, ],
( 'H2' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#ANO631 is 4s 3p 1s on C/N/O and 2s on H
#PMMA_S is the atoms reflected around the plane spanned by
#C-CA and terminating H/next C of next monomer 
        self[("PMMA_S", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : 0.0879100,
( 'CA' , 'dipole') : [ -0.0291200,  0.0296200,  -0.1539700, ],
( 'CA' , 'quadrupole') : [ -2.0245500,  -0.0975300,  0.0353800,  -2.1859700,  0.0402000,  -2.5270200, ],
( 'CA' , 'alpha' ) : [ 4.5794400,  2.4871300,  -1.0627600,  8.3825700,  0.5067000,  4.9749900, ],
( 'CA' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB1', 'charge') : -0.4048900,
( 'CB1', 'dipole') : [ -0.2004100,  0.0922000,  0.0763700, ],
( 'CB1', 'quadrupole') : [ -3.5900300,  -0.1335600,  -0.1238800,  -3.7541600,  0.0458800,  -3.7188700, ],
( 'CB1', 'alpha' ) : [ 10.0188100,  -1.7666100,  -1.9526100,  7.5956500,  1.1467200,  6.8150800, ],
( 'CB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB2', 'charge') : 0.4710400,
( 'CB2', 'dipole') : [ -0.0517200,  -0.0220500,  0.1051800, ],
( 'CB2', 'quadrupole') : [ -1.7657000,  -0.0101100,  -0.0835600,  -3.6450900,  -0.0487500,  -2.5354000, ],
( 'CB2', 'alpha' ) : [ 8.8365500,  0.9066100,  1.4592000,  4.8322600,  0.1031600,  10.9605800, ],
( 'CB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG1', 'charge') : -0.5159100,
( 'OG1', 'dipole') : [ 0.4610200,  -0.0088500,  -0.2393600, ],
( 'OG1', 'quadrupole') : [ -3.5809200,  0.0103800,  -0.3957900,  -4.3029000,  -0.0321700,  -4.5034500, ],
( 'OG1', 'alpha' ) : [ 8.0474600,  0.3255600,  -2.3804000,  3.1514500,  0.1335200,  5.0398200, ],
( 'OG1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG2', 'charge') : -0.3257400,
( 'OG2', 'dipole') : [ -0.4386100,  0.1051500,  0.2283700, ],
( 'OG2', 'quadrupole') : [ -3.3553900,  -0.0033400,  0.3444900,  -3.9565700,  0.2204200,  -3.0354800, ],
( 'OG2', 'alpha' ) : [ 5.3483600,  0.1812600,  3.0544400,  2.6247900,  1.0561100,  9.1920100, ],
( 'OG2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CD' , 'charge') : -0.1590600,
( 'CD' , 'dipole') : [ 0.0263200,  -0.0789800,  -0.2538400, ],
( 'CD' , 'quadrupole') : [ -3.5375000,  -0.0322600,  -0.1174000,  -3.5264400,  0.2115100,  -2.9517300, ],
( 'CD' , 'alpha' ) : [ 5.2463700,  0.2542300,  1.0968300,  6.0198100,  0.9465100,  10.9726600, ],
( 'CD' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'C'  , 'charge') : -0.2430000,
( 'C'  , 'dipole') : [ 0.2119100,  -0.1186200,  0.1950500, ],
( 'C'  , 'quadrupole') : [ -3.3929800,  0.2902500,  0.0041700,  -2.9735900,  0.0625900,  -3.3387700, ],
( 'C'  , 'alpha' ) : [ 8.3201100,  1.9110900,  1.6582300,  9.8336900,  -0.2048900,  5.7786900, ],
( 'C'  , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB1', 'charge') : 0.1388400,
( 'HB1', 'dipole') : [ -0.2183100,  -0.0195700,  -0.2191800, ],
( 'HB1', 'quadrupole') : [ -0.2494600,  0.0490000,  0.3165000,  -0.5654800,  0.0329800,  -0.1924500, ],
( 'HB1', 'alpha' ) : [ 2.1954400,  -0.0400900,  1.3733800,  0.4215900,  0.0230900,  1.4995100, ],
( 'HB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB2', 'charge') : 0.1184500,
( 'HB2', 'dipole') : [ -0.1050100,  -0.1408700,  0.2669800, ],
( 'HB2', 'quadrupole') : [ -0.5270700,  0.1128700,  -0.1582100,  -0.4807900,  -0.2595000,  -0.1312500, ],
( 'HB2', 'alpha' ) : [ 1.1431000,  0.4684700,  -1.3411000,  0.8770300,  -0.8672800,  2.8581000, ],
( 'HB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB3', 'charge') : 0.1261000,
( 'HB3', 'dipole') : [ 0.0591300,  0.2900000,  0.0577200, ],
( 'HB3', 'quadrupole') : [ -0.5273000,  0.1736200,  0.0286000,  0.0385200,  0.0973300,  -0.5307900, ],
( 'HB3', 'alpha' ) : [ 0.6196600,  0.4089100,  0.0230100,  3.2654900,  0.7057000,  0.5300600, ],
( 'HB3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD1', 'charge') : 0.1417500,
( 'HD1', 'dipole') : [ 0.1289400,  0.2106500,  -0.1398600, ],
( 'HD1', 'quadrupole') : [ -0.4291000,  0.1881400,  -0.1206400,  -0.1982600,  -0.1882800,  -0.4740200, ],
( 'HD1', 'alpha' ) : [ 0.7320500,  0.7809300,  -0.6990500,  1.8039300,  -1.4171900,  1.8047900, ],
( 'HD1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD2', 'charge') : 0.1374900,
( 'HD2', 'dipole') : [ 0.1567500,  -0.2432800,  -0.0071400, ],
( 'HD2', 'quadrupole') : [ -0.3827400,  -0.2608600,  -0.0105400,  -0.1534400,  -0.0239000,  -0.6093700, ],
( 'HD2', 'alpha' ) : [ 0.9214000,  -1.1308600,  -0.2684300,  2.6018200,  0.6234100,  0.6634300, ],
( 'HD2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD3', 'charge') : 0.1481100,
( 'HD3', 'dipole') : [ -0.2730200,  -0.0437400,  -0.1055900, ],
( 'HD3', 'quadrupole') : [ 0.0063100,  0.0839300,  0.2048200,  -0.5157500,  0.0363100,  -0.4352400, ],
( 'HD3', 'alpha' ) : [ 3.1243100,  0.5075600,  1.5286400,  0.4349500,  0.2618800,  1.1864700, ],
( 'HD3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H1' , 'charge') : 0.1285100,
( 'H1' , 'dipole') : [ -0.0008900,  -0.0058500,  0.3014200, ],
( 'H1' , 'quadrupole') : [ -0.6034300,  -0.0133800,  -0.0174900,  -0.6296200,  0.0016600,  0.0597100, ],
( 'H1' , 'alpha' ) : [ 0.5325100,  0.0691200,  0.5144800,  0.6369500,  -0.1972000,  3.3780900, ],
( 'H1' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H2' , 'charge') : 0.1427200,
( 'H2' , 'dipole') : [ 0.2474400,  -0.1336400,  -0.1074500, ],
( 'H2' , 'quadrupole') : [ -0.1710600,  -0.2284400,  -0.2150200,  -0.5097300,  0.1247500,  -0.4207200, ],
( 'H2' , 'alpha' ) : [ 2.6045700,  -1.1167300,  -0.5511900,  1.2654500,  0.3536200,  0.6152800, ],
( 'H2' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],

}

#ANO631 is 4s 3p 1s on C/N/O and 2s on H
#PMMA_R is the regular monomer 
        self[("PMMA_R", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : -0.11784,
( 'CA' , 'dipole') : [ -0.63829,  0.16060,  0.29620, ],
( 'CA' , 'quadrupole') : [ -5.82561,  -0.51797,  -1.01054,  -6.68704,  0.09846,  -6.53244, ],
( 'CA' , 'alpha' ) : [ 14.61554,  -2.45780,  -0.49838,  12.54147,  1.54056,  7.72138, ],
( 'CA' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB1', 'charge') : -0.15883,
( 'CB1', 'dipole') : [ 0.10395,  0.17860,  0.16991, ],
( 'CB1', 'quadrupole') : [ -3.48512,  0.15507,  0.15035,  -3.27864,  0.33058,  -3.24950, ],
( 'CB1', 'alpha' ) : [ 7.23509,  2.34605,  1.63406,  8.67598,  1.46970,  6.30975, ],
( 'CB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB2', 'charge') : -0.22919,
( 'CB2', 'dipole') : [ -0.19592,  -0.26389,  0.16885, ],
( 'CB2', 'quadrupole') : [ -2.98513,  -0.20569,  0.25527,  -3.05822,  -0.06614,  -3.21392, ],
( 'CB2', 'alpha' ) : [ 10.94932,  -0.98400,  1.14483,  6.52633,  -1.40518,  5.50276, ],
( 'CB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG1', 'charge') : -0.39930,
( 'OG1', 'dipole') : [ 0.27798,  -0.00544,  -0.00414, ],
( 'OG1', 'quadrupole') : [ -3.03185,  -0.05327,  0.03483,  -3.71108,  -0.01132,  -3.61270, ],
( 'OG1', 'alpha' ) : [ 9.35677,  -1.56462,  0.90268,  5.85534,  -0.30190,  5.10519, ],
( 'OG1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG2', 'charge') : -0.20858,
( 'OG2', 'dipole') : [ -0.25685,  -0.19190,  0.15132, ],
( 'OG2', 'quadrupole') : [ -2.67052,  -0.25557,  0.20509,  -3.11262,  -0.03557,  -3.22374, ],
( 'OG2', 'alpha' ) : [ 8.10260,  -1.28088,  0.89433,  7.82115,  -2.05356,  5.97997, ],
( 'OG2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CD' , 'charge') : 0.11534,
( 'CD' , 'dipole') : [ -0.11618,  -0.28853,  -0.10975, ],
( 'CD' , 'quadrupole') : [ -0.56039,  0.17512,  0.06316,  -0.08250,  0.21871,  -0.48385, ],
( 'CD' , 'alpha' ) : [ 1.40586,  1.27192,  0.69325,  2.59715,  1.07986,  0.93789, ],
( 'CD' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'C'  , 'charge') : -0.34450,
( 'C'  , 'dipole') : [ 0.25249,  -0.37278,  0.05519, ],
( 'C'  , 'quadrupole') : [ -3.37636,  0.66788,  -0.29323,  -3.95910,  0.61088,  -3.38586, ],
( 'C'  , 'alpha' ) : [ 8.24861,  4.86858,  -0.86937,  4.51489,  2.43749,  5.32757, ],
( 'C'  , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB1', 'charge') : 0.13815,
( 'HB1', 'dipole') : [ 0.26331,  0.16179,  -0.02798, ],
( 'HB1', 'quadrupole') : [ -0.08433,  0.29073,  -0.03392,  -0.35588,  -0.00365,  -0.56894, ],
( 'HB1', 'alpha' ) : [ 2.47865,  1.16448,  -0.54223,  1.03173,  -0.28901,  0.61029, ],
( 'HB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB2', 'charge') : 0.11871,
( 'HB2', 'dipole') : [ 0.05771,  -0.30816,  -0.06058, ],
( 'HB2', 'quadrupole') : [ -0.55740,  -0.10676,  -0.02422,  0.03062,  0.11497,  -0.61478, ],
( 'HB2', 'alpha' ) : [ 0.60588,  -0.72262,  -0.23409,  3.31004,  1.11645,  0.97344, ],
( 'HB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB3', 'charge') : 0.12646,
( 'HB3', 'dipole') : [ -0.20457,  0.10440,  -0.19673, ],
( 'HB3', 'quadrupole') : [ -0.22809,  -0.17347,  0.29032,  -0.45762,  -0.14236,  -0.33774, ],
( 'HB3', 'alpha' ) : [ 1.74464,  -0.67842,  1.31591,  0.66376,  -0.62109,  2.02593, ],
( 'HB3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD1', 'charge') : 0.14170,
( 'HD1', 'dipole') : [ -0.15437,  0.23706,  -0.02334, ],
( 'HD1', 'quadrupole') : [ -0.37117,  -0.27556,  0.03641,  -0.20740,  -0.06965,  -0.52315, ],
( 'HD1', 'alpha' ) : [ 0.83023,  -1.06693,  0.06319,  3.18367,  -0.07987,  0.32228, ],
( 'HD1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD2', 'charge') : 0.13724,
( 'HD2', 'dipole') : [ -0.01135,  -0.10761,  0.26867, ],
( 'HD2', 'quadrupole') : [ -0.57231,  -0.01704,  -0.04711,  -0.48482,  -0.22774,  -0.08893, ],
( 'HD2', 'alpha' ) : [ 0.45002,  -0.01151,  0.41378,  0.58290,  -0.58466,  3.16006, ],
( 'HD2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD3', 'charge') : 0.14818,
( 'HD3', 'dipole') : [ 0.27912,  0.04822,  -0.08576, ],
( 'HD3', 'quadrupole') : [ 0.03042,  0.10087,  -0.16966,  -0.50489,  -0.02303,  -0.46999, ],
( 'HD3', 'alpha' ) : [ 3.64072,  0.91835,  -0.72085,  0.61695,  -0.20477,  0.48766, ],
( 'HD3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H1' , 'charge') : 0.58073,
( 'H1' , 'dipole') : [ -0.07841,  0.12404,  -0.27410, ],
( 'H1' , 'quadrupole') : [ 0.38383,  -0.03725,  -0.30779,  0.60251,  -0.07269,  1.47366, ],
( 'H1' , 'alpha' ) : [ -1.25650,  0.06600,  0.06955,  1.92222,  0.50057,  3.68812, ],
( 'H1' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H2' , 'charge') : -0.04825,
( 'H2' , 'dipole') : [ 0.48391,  0.47023,  -0.27795, ],
( 'H2' , 'quadrupole') : [ -1.61985,  0.38907,  -0.30204,  -0.91710,  0.33424,  -0.82184, ],
( 'H2' , 'alpha' ) : [ -0.91085,  1.52225,  -1.84851,  1.83510,  2.96751,  5.48240, ],
( 'H2' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],

}

#ANO631 is 4s 3p 1s on C/N/O and 2s on H
#PMMA_N is first monomer with lacks last hydrogen
        self[("PMMA_C", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : 0.09090,
( 'CA' , 'dipole') : [ -0.02544,  0.04033,  -0.15658, ],
( 'CA' , 'quadrupole') : [ -1.94723,  -0.14191,  -0.00229,  -2.14520,  0.04847,  -2.64436, ],
( 'CA' , 'alpha' ) : [ 4.01482,  3.35435,  -0.84809,  8.93336,  0.32187,  5.41772, ],
( 'CA' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB1', 'charge') : -0.40628,
( 'CB1', 'dipole') : [ -0.20055,  0.09286,  0.07620, ],
( 'CB1', 'quadrupole') : [ -3.59732,  -0.12807,  -0.12315,  -3.76400,  0.04551,  -3.72279, ],
( 'CB1', 'alpha' ) : [ 10.02339,  -1.79021,  -1.96708,  7.59906,  1.15400,  6.84889, ],
( 'CB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CB2', 'charge') : 0.47170,
( 'CB2', 'dipole') : [ -0.05208,  -0.02350,  0.10393, ],
( 'CB2', 'quadrupole') : [ -1.76455,  -0.01104,  -0.08033,  -3.63993,  -0.04871,  -2.53012, ],
( 'CB2', 'alpha' ) : [ 8.82717,  0.90321,  1.49064,  4.80070,  0.12514,  10.89934, ],
( 'CB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG1', 'charge') : -0.51611,
( 'OG1', 'dipole') : [ 0.46126,  -0.00885,  -0.23994, ],
( 'OG1', 'quadrupole') : [ -3.58007,  0.01058,  -0.39657,  -4.30117,  -0.03033,  -4.50021, ],
( 'OG1', 'alpha' ) : [ 8.02344,  0.32047,  -2.35870,  3.14336,  0.12946,  5.02574, ],
( 'OG1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'OG2', 'charge') : -0.32499,
( 'OG2', 'dipole') : [ -0.43823,  0.10473,  0.22834, ],
( 'OG2', 'quadrupole') : [ -3.35410,  -0.00336,  0.34459,  -3.95608,  0.22198,  -3.03722, ],
( 'OG2', 'alpha' ) : [ 5.35716,  0.18922,  3.05916,  2.62060,  1.05770,  9.18009, ],
( 'OG2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'CD' , 'charge') : -0.15883,
( 'CD' , 'dipole') : [ 0.02656,  -0.07893,  -0.25405, ],
( 'CD' , 'quadrupole') : [ -3.53730,  -0.03205,  -0.11732,  -3.52650,  0.21177,  -2.95011, ],
( 'CD' , 'alpha' ) : [ 5.24622,  0.25709,  1.10133,  6.02255,  0.94589,  10.95941, ],
( 'CD' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'C'  , 'charge') : -0.24058,
( 'C'  , 'dipole') : [ 0.20940,  -0.13166,  0.19645, ],
( 'C'  , 'quadrupole') : [ -3.38882,  0.31223,  -0.00687,  -2.94892,  0.02443,  -3.33665, ],
( 'C'  , 'alpha' ) : [ 7.83192,  2.67407,  1.80058,  10.35335,  -0.29867,  6.20996, ],
( 'C'  , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB1', 'charge') : 0.13822,
( 'HB1', 'dipole') : [ -0.21857,  -0.01962,  -0.21951, ],
( 'HB1', 'quadrupole') : [ -0.24996,  0.04889,  0.31694,  -0.56632,  0.03302,  -0.19236, ],
( 'HB1', 'alpha' ) : [ 2.19358,  -0.04313,  1.37299,  0.41904,  0.02058,  1.50332, ],
( 'HB1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB2', 'charge') : 0.11873,
( 'HB2', 'dipole') : [ -0.10416,  -0.14163,  0.26660, ],
( 'HB2', 'quadrupole') : [ -0.52858,  0.11444,  -0.15773,  -0.48197,  -0.25997,  -0.13081, ],
( 'HB2', 'alpha' ) : [ 1.14095,  0.47324,  -1.34083,  0.87376,  -0.87849,  2.87424, ],
( 'HB2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HB3', 'charge') : 0.12649,
( 'HB3', 'dipole') : [ 0.06007,  0.29071,  0.05763, ],
( 'HB3', 'quadrupole') : [ -0.53014,  0.17215,  0.02862,  0.03798,  0.09765,  -0.53110, ],
( 'HB3', 'alpha' ) : [ 0.62410,  0.41195,  0.01901,  3.27262,  0.70892,  0.53573, ],
( 'HB3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD1', 'charge') : 0.14172,
( 'HD1', 'dipole') : [ 0.12895,  0.21061,  -0.13987, ],
( 'HD1', 'quadrupole') : [ -0.42902,  0.18810,  -0.12059,  -0.19852,  -0.18818,  -0.47415, ],
( 'HD1', 'alpha' ) : [ 0.73178,  0.78052,  -0.69845,  1.80239,  -1.41586,  1.80508, ],
( 'HD1', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD2', 'charge') : 0.13732,
( 'HD2', 'dipole') : [ 0.15682,  -0.24337,  -0.00719, ],
( 'HD2', 'quadrupole') : [ -0.38278,  -0.26089,  -0.01051,  -0.15370,  -0.02396,  -0.60917, ],
( 'HD2', 'alpha' ) : [ 0.92010,  -1.13029,  -0.26659,  2.60377,  0.62146,  0.66646, ],
( 'HD2', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'HD3', 'charge') : 0.14819,
( 'HD3', 'dipole') : [ -0.27296,  -0.04375,  -0.10558, ],
( 'HD3', 'quadrupole') : [ 0.00628,  0.08393,  0.20481,  -0.51562,  0.03631,  -0.43523, ],
( 'HD3', 'alpha' ) : [ 3.12562,  0.50952,  1.52823,  0.43551,  0.26265,  1.18563, ],
( 'HD3', 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H1' , 'charge') : 0.11813,
( 'H1' , 'dipole') : [ -0.00069,  -0.00491,  0.30469, ],
( 'H1' , 'quadrupole') : [ -0.61446,  -0.01316,  -0.01685,  -0.64178,  0.01695,  0.00745, ],
( 'H1' , 'alpha' ) : [ 0.51853,  0.10710,  0.48346,  0.67469,  -0.16342,  3.92166, ],
( 'H1' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
( 'H2' , 'charge') : 0.15568,
( 'H2' , 'dipole') : [ 0.24367,  -0.13543,  -0.10613, ],
( 'H2' , 'quadrupole') : [ -0.13979,  -0.26009,  -0.21920,  -0.46053,  0.13594,  -0.40496, ],
( 'H2' , 'alpha' ) : [ 2.05913,  -0.82942,  -0.31819,  1.16610,  0.24148,  0.51192, ],
( 'H2' , 'beta'  ) : [ 0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000,  0.00000, ],
}

#Tuomas version is the spiral chain he generated in email sent aug-2015 to kth
        self[("PMMA_TUOMAS", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : 0.0443400,
( 'CA' , 'dipole') : [ -0.0213700,  0.0334600,  -0.1598300, ],
( 'CA' , 'quadrupole') : [ -2.3457300,  -0.4491400,  -0.0025800,  -2.5724700,  -0.0310200,  -2.7638700, ],
( 'CA' , 'alpha' ) : [ 8.1566400,  3.3347300,  0.0509700,  9.0868500,  0.0307600,  6.7571000, ],
( 'CA' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'C'  , 'charge') : -0.2257000,
( 'C'  , 'dipole') : [ 0.0723600,  -0.1305400,  0.0688900, ],
( 'C'  , 'quadrupole') : [ -2.9842700,  0.4596100,  0.1417900,  -3.2578800,  0.1007800,  -3.5571100, ],
( 'C'  , 'alpha' ) : [ 11.7773600,  4.5534200,  2.2273300,  9.6355000,  0.3508400,  5.2796900, ],
( 'C'  , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB1', 'charge') : -0.3864100,
( 'CB1', 'dipole') : [ -0.1396700,  0.1421300,  0.0827100, ],
( 'CB1', 'quadrupole') : [ -3.7119000,  -0.2191200,  -0.1228800,  -3.7095300,  0.0908500,  -3.7540700, ],
( 'CB1', 'alpha' ) : [ 8.8670000,  -2.1998500,  -1.3637500,  8.7618500,  1.5091100,  7.2184700, ],
( 'CB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB2', 'charge') : 0.5018900,
( 'CB2', 'dipole') : [ -0.0029000,  -0.0017100,  0.1414200, ],
( 'CB2', 'quadrupole') : [ -1.9858400,  -0.0456700,  -0.3221200,  -3.5563100,  0.0050600,  -2.3840200, ],
( 'CB2', 'alpha' ) : [ 8.1551400,  0.2584800,  1.1234300,  4.0510200,  -0.1872600,  10.3125900, ],
( 'CB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CD' , 'charge') : -0.1603000,
( 'CD' , 'dipole') : [ -0.0101300,  -0.0146800,  -0.2826200, ],
( 'CD' , 'quadrupole') : [ -3.5795900,  -0.0073900,  -0.0868800,  -3.6299900,  0.0403900,  -2.9316900, ],
( 'CD' , 'alpha' ) : [ 5.8584400,  0.0086400,  1.8009500,  6.0965100,  -0.0203700,  11.0227000, ],
( 'CD' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG1', 'charge') : -0.4722100,
( 'OG1', 'dipole') : [ 0.4513200,  -0.0054600,  -0.3151200, ],
( 'OG1', 'quadrupole') : [ -3.6749700,  -0.0235800,  -0.5274100,  -4.0852000,  -0.0105400,  -4.0988800, ],
( 'OG1', 'alpha' ) : [ 6.6790900,  0.1733300,  -2.2041700,  2.6694700,  0.0311000,  5.3616800, ],
( 'OG1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG2', 'charge') : -0.3456000,
( 'OG2', 'dipole') : [ -0.4086400,  0.0116500,  0.3209200, ],
( 'OG2', 'quadrupole') : [ -3.3159900,  -0.0019700,  0.3333100,  -4.0288400,  0.0346600,  -3.1371900, ],
( 'OG2', 'alpha' ) : [ 5.8661500,  0.0176000,  3.0846200,  2.3240200,  0.0215500,  8.6023900, ],
( 'OG2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB1', 'charge') : 0.1345300,
( 'HB1', 'dipole') : [ -0.2885100,  -0.0875700,  -0.0774400, ],
( 'HB1', 'quadrupole') : [ 0.0171700,  0.1796400,  0.1659200,  -0.5539900,  0.0397500,  -0.5181700, ],
( 'HB1', 'alpha' ) : [ 3.6414800,  0.4843000,  0.5883100,  0.5790200,  0.1306900,  0.5176300, ],
( 'HB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB2', 'charge') : 0.1229800,
( 'HB2', 'dipole') : [ 0.0300100,  0.0028500,  0.3117500, ],
( 'HB2', 'quadrupole') : [ -0.6084000,  0.0178700,  0.0782000,  -0.6063500,  -0.0049400,  0.0784900, ],
( 'HB2', 'alpha' ) : [ 0.4815500,  -0.0597700,  -0.1604700,  0.5271700,  0.4961900,  4.0710500, ],
( 'HB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB3', 'charge') : 0.1271300,
( 'HB3', 'dipole') : [ 0.0699800,  0.2808900,  -0.1212100, ],
( 'HB3', 'quadrupole') : [ -0.5785000,  0.1440600,  -0.0476000,  -0.0351100,  -0.2590600,  -0.4900300, ],
( 'HB3', 'alpha' ) : [ 0.5396400,  0.2899500,  -0.1007400,  3.5992700,  -0.9054800,  0.6961800, ],
( 'HB3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD1', 'charge') : 0.1332100,
( 'HD1', 'dipole') : [ 0.1194700,  0.2331400,  -0.1007100, ],
( 'HD1', 'quadrupole') : [ -0.4627000,  0.2051300,  -0.0980100,  -0.1502200,  -0.1349100,  -0.5510800, ],
( 'HD1', 'alpha' ) : [ 0.6888900,  0.8270500,  -0.4896500,  2.5006300,  -1.4348500,  1.3992300, ],
( 'HD1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD2', 'charge') : 0.1332300,
( 'HD2', 'dipole') : [ 0.1351400,  -0.2313300,  -0.0865100, ],
( 'HD2', 'quadrupole') : [ -0.4312800,  -0.2317100,  -0.0945100,  -0.1672400,  0.1111500,  -0.5734400, ],
( 'HD2', 'alpha' ) : [ 0.8108100,  -0.9547000,  -0.5165300,  2.4531700,  1.2580800,  1.2346100, ],
( 'HD2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD3', 'charge') : 0.1433700,
( 'HD3', 'dipole') : [ -0.2877100,  -0.0112800,  -0.0680500, ],
( 'HD3', 'quadrupole') : [ 0.0832800,  0.0227700,  0.1323400,  -0.5416900,  0.0066500,  -0.5048300, ],
( 'HD3', 'alpha' ) : [ 3.7400800,  0.1102100,  1.1767500,  0.3687500,  0.0310000,  0.7811800, ],
( 'HD3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H1' , 'charge') : 0.1384100,
( 'H1' , 'dipole') : [ 0.1696700,  -0.1941800,  -0.1476700, ],
( 'H1' , 'quadrupole') : [ -0.3753400,  -0.2734300,  -0.2014900,  -0.3183200,  0.2257200,  -0.3939100, ],
( 'H1' , 'alpha' ) : [ 1.5160300,  -0.8662700,  -0.4436900,  1.8233200,  0.7030500,  0.7008800, ],
( 'H1' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H2' , 'charge') : 0.1116000,
( 'H2' , 'dipole') : [ 0.0009600,  -0.0818700,  0.2989800, ],
( 'H2' , 'quadrupole') : [ -0.6292300,  -0.0165100,  0.0035000,  -0.5888100,  -0.1613400,  0.0053100, ],
( 'H2' , 'alpha' ) : [ 0.6942400,  0.1414100,  0.3273400,  0.9640900,  -0.9638500,  3.1848300, ],
( 'H2' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
        }

#Tuomas version but with 7 digit precision generated sept 19 2015
        self[("PMMA_TUOMAS7", "B3LYP", "ANO631", True, "0.0")] = \
{
'meta' : { 'label' : 'pdb_name', },
( 'CA' , 'charge') : 0.0440759,
( 'CA' , 'dipole') : [ 0.1090944,  0.2662727,  0.0508053, ],
( 'CA' , 'quadrupole') : [ -2.5494818,  -0.3577929,  -0.1242853,  -2.3151580,  -0.0457289,  -2.8191007, ],
( 'CA' , 'alpha' ) : [ 8.6293399,  2.4030613,  -0.0112105,  8.8102798,  0.4799834,  6.5123963, ],
( 'CA' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'C'  , 'charge') : -0.2258010,
( 'C'  , 'dipole') : [ 0.1457685,  -0.3048847,  0.1232080, ],
( 'C'  , 'quadrupole') : [ -2.7413011,  0.2483185,  0.2418958,  -3.5111143,  0.1331555,  -3.5483494, ],
( 'C'  , 'alpha' ) : [ 13.8737762,  1.8002515,  3.3269360,  6.7446055,  -0.2345832,  6.0438351, ],
( 'C'  , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB1', 'charge') : -0.3864351,
( 'CB1', 'dipole') : [ -0.1396741,  0.1422125,  0.0827263, ],
( 'CB1', 'quadrupole') : [ -3.7110440,  -0.2199026,  -0.1233561,  -3.7099986,  0.0907019,  -3.7534996, ],
( 'CB1', 'alpha' ) : [ 8.8638214,  -2.1967208,  -1.3626493,  8.7602282,  1.5074942,  7.2152274, ],
( 'CB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CB2', 'charge') : 0.5018671,
( 'CB2', 'dipole') : [ -0.0028558,  -0.0014875,  0.1413563, ],
( 'CB2', 'quadrupole') : [ -1.9847498,  -0.0460366,  -0.3222159,  -3.5563431,  0.0052778,  -2.3846960, ],
( 'CB2', 'alpha' ) : [ 8.1594762,  0.2584125,  1.1230158,  4.0520191,  -0.1870143,  10.3118252, ],
( 'CB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'CD' , 'charge') : -0.1603673,
( 'CD' , 'dipole') : [ -0.0102256,  -0.0146855,  -0.2824173, ],
( 'CD' , 'quadrupole') : [ -3.5797702,  -0.0072089,  -0.0877466,  -3.6300683,  0.0398228,  -2.9310944, ],
( 'CD' , 'alpha' ) : [ 5.8590465,  0.0073705,  1.8017502,  6.0936976,  -0.0204325,  11.0264882, ],
( 'CD' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG1', 'charge') : -0.4721779,
( 'OG1', 'dipole') : [ 0.4512407,  -0.0056319,  -0.3150539, ],
( 'OG1', 'quadrupole') : [ -3.6744980,  -0.0238309,  -0.5273902,  -4.0853855,  -0.0103638,  -4.0993332, ],
( 'OG1', 'alpha' ) : [ 6.6806009,  0.1721331,  -2.2048750,  2.6697124,  0.0320404,  5.3614037, ],
( 'OG1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'OG2', 'charge') : -0.3456170,
( 'OG2', 'dipole') : [ -0.4086253,  0.0116525,  0.3207024, ],
( 'OG2', 'quadrupole') : [ -3.3158473,  -0.0019870,  0.3324995,  -4.0295303,  0.0344687,  -3.1376199, ],
( 'OG2', 'alpha' ) : [ 5.8675918,  0.0173718,  3.0828076,  2.3248470,  0.0211407,  8.6044652, ],
( 'OG2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB1', 'charge') : 0.1345330,
( 'HB1', 'dipole') : [ -0.2885982,  -0.0874386,  -0.0774255, ],
( 'HB1', 'quadrupole') : [ 0.0174849,  0.1792084,  0.1658382,  -0.5541902,  0.0396063,  -0.5181651, ],
( 'HB1', 'alpha' ) : [ 3.6425540,  0.4825448,  0.5877210,  0.5780623,  0.1301629,  0.5171767, ],
( 'HB1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB2', 'charge') : 0.1230228,
( 'HB2', 'dipole') : [ 0.0299944,  0.0029419,  0.3117759, ],
( 'HB2', 'quadrupole') : [ -0.6082256,  0.0178620,  0.0780501,  -0.6061291,  -0.0047454,  0.0783025, ],
( 'HB2', 'alpha' ) : [ 0.4811986,  -0.0597645,  -0.1609318,  0.5269372,  0.4965153,  4.0678316, ],
( 'HB2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HB3', 'charge') : 0.1270854,
( 'HB3', 'dipole') : [ 0.0700239,  0.2809632,  -0.1212984, ],
( 'HB3', 'quadrupole') : [ -0.5786021,  0.1441305,  -0.0477002,  -0.0350896,  -0.2593352,  -0.4899704, ],
( 'HB3', 'alpha' ) : [ 0.5398364,  0.2902576,  -0.1008905,  3.6005671,  -0.9065057,  0.6969241, ],
( 'HB3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD1', 'charge') : 0.1333215,
( 'HD1', 'dipole') : [ 0.1194970,  0.2330639,  -0.1007282, ],
( 'HD1', 'quadrupole') : [ -0.4623922,  0.2048743,  -0.0980788,  -0.1505537,  -0.1349773,  -0.5504641, ],
( 'HD1', 'alpha' ) : [ 0.6883192,  0.8261191,  -0.4897140,  2.4951131,  -1.4330305,  1.3984175, ],
( 'HD1', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD2', 'charge') : 0.1332854,
( 'HD2', 'dipole') : [ 0.1351277,  -0.2314224,  -0.0864112, ],
( 'HD2', 'quadrupole') : [ -0.4312046,  -0.2317623,  -0.0944398,  -0.1668467,  0.1110812,  -0.5734305, ],
( 'HD2', 'alpha' ) : [ 0.8106587,  -0.9546795,  -0.5160900,  2.4527943,  1.2568635,  1.2332579, ],
( 'HD2', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'HD3', 'charge') : 0.1433251,
( 'HD3', 'dipole') : [ -0.2877301,  -0.0113053,  -0.0680810, ],
( 'HD3', 'quadrupole') : [ 0.0836255,  0.0228797,  0.1324559,  -0.5419697,  0.0066653,  -0.5051359, ],
( 'HD3', 'alpha' ) : [ 3.7439489,  0.1104941,  1.1782992,  0.3691644,  0.0310191,  0.7822377, ],
( 'HD3', 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H1' , 'charge') : 0.1383814,
( 'H1' , 'dipole') : [ 0.0903320,  -0.1623805,  -0.2393736, ],
( 'H1' , 'quadrupole') : [ -0.5226408,  -0.1290455,  -0.1787396,  -0.4016111,  0.2716690,  -0.1634273, ],
( 'H1' , 'alpha' ) : [ 0.6987583,  -0.4682892,  -0.4642981,  1.4887586,  1.0405455,  1.8544082, ],
( 'H1' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
( 'H2' , 'charge') : 0.1115007,
( 'H2' , 'dipole') : [ -0.0301133,  -0.1972351,  0.2362922, ],
( 'H2' , 'quadrupole') : [ -0.6289640,  0.0486618,  -0.0399327,  -0.3800949,  -0.2995246,  -0.2040592, ],
( 'H2' , 'alpha' ) : [ 0.7225654,  -0.0374793,  0.1612044,  1.7902485,  -2.0314522,  2.3380008, ],
( 'H2' , 'beta'  ) : [ 0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000,  0.0000000, ],
}
